import pycuda.autoinit as autoinit
import pycuda.driver as drv
import pycuda
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
import bisect
import numpy, pylab, time
from matplotlib.cm import jet
try:
    from pycuda.compiler import SourceModule
except ImportError:
    from pycuda.driver import SourceModule

N = 10000
blocksize = 256 # too many registers for 512
doplot = True
kernel_run_length = None # set to None to make it the maximum

N = int(numpy.ceil(1. * N / blocksize) * blocksize)

if drv.get_version() == (2, 0, 0): # cuda version
    precision = 'float'
elif drv.get_version() > (2, 0, 0):
    precision = 'double'
else:
    raise Exception, "CUDA 2.0 required"

if precision == 'double':
    mydtype = numpy.float64
else:
    mydtype = numpy.float32

block = (blocksize, 1, 1)
grid = (N / blocksize, 1)

mod = SourceModule("""
__global__ void runsim(
                    int Tstart, int Tend,     // Start, end time as integer (t=T*dt)
                    SCALAR *V_arr,            // State variables for each neuron
                    SCALAR *tau_arr,
                    SCALAR *R_arr,
                    SCALAR *I_arr,            // Input current
                    int *spikecount,          // Number of spikes produced by each neuron
                    int *num_coincidences,    // Count of coincidences for each neuron
                    int *spiketimes,          // Array of all spike times as integers
                    int *spiketime_indices    // Pointer into above array for each neuron
                    )
{
    const int neuron_index = blockIdx.x * blockDim.x + threadIdx.x;
    // Load variables at start
    SCALAR V = V_arr[neuron_index];
    SCALAR tau = tau_arr[neuron_index];
    SCALAR R = R_arr[neuron_index];
    int spiketime_index = spiketime_indices[neuron_index];
    int last_spike_time = spiketimes[spiketime_index];
    int next_spike_time = spiketimes[spiketime_index+1];
    int ncoinc = num_coincidences[neuron_index];
    int nspikes = spikecount[neuron_index];
    for(int T=Tstart; T<Tend; T++)
    {
        SCALAR t = T*_dt_;
        // Read input current
        SCALAR I = I_arr[T]; // this is a global read for each thread, can maybe
                             // reduce this by having just one read per block,
                             // put it into shared memory, and then have all the
                             // threads in that block read it, we could even
                             // maybe buffer reads of I into shared memory -
                             // experiment with this 
        // State update
        SCALAR V__tmp = (-V+R*I)/tau;
        V += _dt_*V__tmp;
        // Threshold
        const bool has_spiked = V>1.;     // Vt=1
        nspikes += has_spiked;
        // Reset
        //if(has_spiked)
            V = V*!has_spiked;            // Vr=0
        // Coincidence counter
        ncoinc += has_spiked && (((last_spike_time+_delta_)>=T) || ((next_spike_time-_delta_)<=T));
        if(T==next_spike_time){           // divergence for each input spike
            spiketime_index++;
            last_spike_time = next_spike_time;
            next_spike_time = spiketimes[spiketime_index+1];
        }
    }
    // Store variables at end
    V_arr[neuron_index] = V;
    spiketime_indices[neuron_index] = spiketime_index;
    num_coincidences[neuron_index] = ncoinc;
    spikecount[neuron_index] = nspikes;
}
""".replace('SCALAR', precision).replace('_dt_', '0.0001').replace('_delta_', '40'))
runsim = mod.get_function("runsim")

# Read data from files
I = numpy.loadtxt('current_long.txt')
duration = len(I)
if kernel_run_length is None:
    kernel_run_length = duration
spiketimes = numpy.array((numpy.loadtxt('spikes_long.txt') - 28.) / 0.0001, dtype=int)
# we add a -10000 at the beginning as there is no previous spike at the beginning,
# and one at the end because the kernel moves to the next spike by checking for the
# existence of the current spike 
spiketimes = numpy.hstack((-10000, spiketimes, -10000))

V = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))
R = gpuarray.to_gpu(numpy.array(numpy.random.rand(N) * 2e9 + 1e9, dtype=mydtype))
tau = gpuarray.to_gpu(numpy.array(numpy.random.rand(N) * 0.050 + 0.001, dtype=mydtype))
I = gpuarray.to_gpu(numpy.array(I, dtype=mydtype))
num_coincidences = gpuarray.to_gpu(numpy.zeros(N, dtype=int))
spiketime_indices = gpuarray.to_gpu(numpy.zeros(N, dtype=int))
spiketimes = gpuarray.to_gpu(numpy.array(spiketimes, dtype=int))
spikecount = gpuarray.to_gpu(numpy.zeros(N, dtype=int))

#runsim.prepare(('i', 'i', 'i', 'i'), block)
#stateupdate_args = (grid, int(V.gpudata), int(ge.gpudata), int(gi.gpudata))
#stateupdate.prepared_call(*stateupdate_args)

start_time = time.time()
for i in xrange(duration / kernel_run_length):
    runsim(
           numpy.int32(i * kernel_run_length), numpy.int32((i + 1) * kernel_run_length),
           V, tau, R,
           I,
           spikecount,
           num_coincidences,
           spiketimes,
           spiketime_indices,
           block=block, grid=grid
           )
    autoinit.context.synchronize()
running_time = time.time() - start_time

print 'N:', N, 'blocksize:', blocksize
print 'Duration:', duration, 'steps =', duration * 0.0001, 's'
print 'Kernel run length:', kernel_run_length, 'steps =', kernel_run_length * 0.0001, 's'
print 'Total running time:', running_time

if doplot:
    spikecount = spikecount.get()
    num_coincidences = num_coincidences.get()
    R = R.get()
    tau = tau.get()
    print 'Spike count varies between', spikecount.min(), 'and', spikecount.max()
    print 'Num coincidences varies between', num_coincidences.min(), 'and', num_coincidences.max()

    pylab.subplot(221)
    pylab.scatter(R, tau, color=jet(spikecount / float(spikecount.max())))
    pylab.xlabel('R')
    pylab.ylabel('tau')
    pylab.title('Spike count, max = ' + str(spikecount.max()))
    pylab.axis('tight')

    pylab.subplot(222)
    pylab.scatter(R, tau, color=jet(num_coincidences / float(num_coincidences.max())))
    pylab.xlabel('R')
    pylab.ylabel('tau')
    pylab.title('Num coincidences, max = ' + str(num_coincidences.max()))
    pylab.axis('tight')

    spikecount -= num_coincidences
    num_coincidences -= spikecount
    num_coincidences[num_coincidences < 0] = 0
    maxcoinc = num_coincidences.max()
    num_coincidences = (1. * num_coincidences) / maxcoinc

    pylab.subplot(212)
    pylab.scatter(R, tau, color=jet(num_coincidences))
    pylab.xlabel('R')
    pylab.ylabel('tau')
    pylab.title('Hot = ' + str(maxcoinc) + ' excess coincidences, cool = 0 or less')
    pylab.axis('tight')

    pylab.show()



from brian import *
from time import time

m = 3
k = 1 # number of time steps


class Fake(float):
    pass
    #def __init__(self,value):
    #    #super(Fake,self).__init__(value)
    #    #self.x=value
    #    pass

if m == 1:
    t1 = time()
    N = 10000000
    for _ in xrange(N):
        x = 5.
    t2 = time()
    print k * (t2 - t1) * second / N
elif m == 2:
    t1 = time()
    N = 100000
    for _ in xrange(N):
        #x=1*second
        #x=second.copy()
        #x=Quantity(2.)
        #x=Fake()
        #x.dim=D
        x = Quantity.with_dimensions(2., second.dim)
    t2 = time()
    print k * (t2 - t1) * second / N
elif m == 3:
    t1 = time()
    N = 100000
    for _ in xrange(N):
        x = Fake(5.)
    t2 = time()
    print k * (t2 - t1) * second / N



#!/usr/bin/env python

"""
setup.py file for SWIG example
"""

from distutils.core import setup, Extension
import os, numpy
numpy_base_dir = os.path.split(numpy.__file__)[0]
numpy_include_dir = os.path.join(numpy_base_dir, 'core/include')

fastexp_module = Extension('_fastexp',
                           sources=['fastexp_wrap.cxx',
                                    'fastexp.cpp',
                                    'fexp.c'],
                           include_dirs=[numpy_include_dir],
                           extra_compile_args=['-O3']
                           )

setup (name='fastexp',
       version='0.1',
       author="Dan Goodman",
       description="""Simple swig example from docs""",
       ext_modules=[fastexp_module],
       py_modules=["fastexp"],
       )



from brian import *
from hrtf import *
from ..prefs import get_samplerate
from ..filtering.fractionaldelay import FractionalDelay
from ..sounds import silence

__all__ = ['HeadlessDatabase']

speed_of_sound_in_air = 343.2*metre/second

class HeadlessDatabase(HRTFDatabase):
    '''
    Database for creating HRTFSet with artificial interaural time-differences
    
    Initialisation keywords:
    
    ``n``, ``azim_max``, ``diameter``
        Specify the ITDs for two ears separated by distance ``diameter`` with
        no head. ITDs corresponding to ``n`` angles equally spaced between
        ``-azim_max`` and ``azim_max`` are used. The default diameter is that
        which gives the maximum ITD as 650 microseconds. The ITDs are computed
        with the formula ``diameter*sin(azim)/speed_of_sound_in_air``. In this
        case, the generated :class:`HRTFSet` will have coordinates of ``azim``
        and ``itd``.
    ``itd``
        Instead of specifying the keywords above, just give the ITDs directly.
        In this case, the generated :class:`HRTFSet` will have coordinates of
        ``itd`` only.
    ``fractional_itds=False``
        Set this to ``True`` to allow ITDs with a fractional multiple of the
        timestep ``1/samplerate``. Note that the filters used to do this are
        not perfect and so this will introduce a small amount of numerical
        error, and so shouldn't be used unless this level of timing precision
        is required. See :class:`FractionalDelay` for more details.
        
    To get the HRTFSet, the simplest thing to do is just::
    
        hrtfset = HeadlessDatabase(13).load_subject()
        
    The generated ITDs can be returned using the ``itd`` attribute of the
    :class:`HeadlessDatabase` object.
    
    If ``fractional_itds=False`` then    
    Note that the delays induced in the left and right channels are not
    symmetric as making them so wastes half the samplerate (if the delay to
    the left channel is itd/2 and the delay to the right channel is -itd/2).
    Instead, for each channel either the left channel delay is 0 and the right
    channel delay is -itd (if itd<0) or the left channel delay is itd and the
    right channel delay is 0 (if itd>0).
    
    If ``fractional_itds=True`` then delays in the left and right channels will
    be symmetric around a global offset of ``delay_offset``.
    '''
    def __init__(self, n=None, azim_max=pi/2,
                 diameter=speed_of_sound_in_air*650*usecond,
                 itd=None, samplerate=None, fractional_itds=False):
        if itd is None:
            azim = linspace(-azim_max, azim_max, n)
            itd = diameter*sin(azim)/speed_of_sound_in_air
            coords = make_coordinates(azim=azim, itd=itd)
        else:
            coords = make_coordinates(itd=itd)
        self.itd = itd
        samplerate = self.samplerate = get_samplerate(samplerate)
        if not fractional_itds:
            dl = itd.copy()
            dr = -itd
            dl[dl<0] = 0
            dr[dr<0] = 0
            dl = array(rint(dl*samplerate), dtype=int)
            dr = array(rint(dr*samplerate), dtype=int)
            idxmax = max(amax(dl), amax(dr))
            data = zeros((2, len(itd), idxmax+1))
            data[0, arange(len(itd)), dl] = 1
            data[1, arange(len(itd)), dr] = 1
        else:
            delays = hstack((itd/2, -itd/2))
            fd = FractionalDelay(silence(1*ms, samplerate=samplerate), delays)
            ir = fd.impulse_response
            data = zeros((2, len(itd), fd.filter_length))
            data[0, :, :] = ir[:len(itd), :]
            data[1, :, :] = ir[len(itd):, :]
            self.delay_offset = fd.delay_offset
        self.hrtfset = HRTFSet(data, samplerate, coords)
        self.hrtfset.name = 'ITDDatabaseSubject'
        self.subjects = ['0']

    def load_subject(self, subject='0'):
        return self.hrtfset



#from ..log import *
import atexit

__all__ = ['initialise_cuda', 'set_gpu_device', 'close_cuda']
    
try:
    import pycuda
    import pycuda.driver as drv
    pycuda.context = None
    pycuda.isinitialised = False
    MAXGPU = 0 # drv.Device.count()
#    
    def initialise_cuda():
        global MAXGPU
        if  not pycuda.isinitialised:
            drv.init()
            pycuda.isinitialised = True
            MAXGPU = drv.Device.count()
    
    def set_gpu_device(n):
        """
        This function makes pycuda use GPU number n in the system.
        """
        initialise_cuda()
        #log_debug('brian.hears', "Setting PyCUDA context number %d" % n)
        try:
            pycuda.context.detach()
        except:
            pass
        pycuda.context = drv.Device(n).make_context()
    
    def close_cuda():
        """
        Closes the current context. MUST be called at the end of the script.
        """
        if pycuda.context is not None:
            #log_debug('brian.hears', "Closing current PyCUDA context")
            try:
                pycuda.context.pop()
                pycuda.context = None
            except:
                pass
    
    atexit.register(close_cuda)

except:
    MAXGPU = 0

    def initialise_cuda():
        pass
    
    def set_gpu_device(n):
        raise Exception("PyCUDA not available")
        pass

    def close_cuda():
        pass

#if __name__ == '__main__':
#    close_cuda()


from brian import *

tau_c=100*ms
tau=10*ms
w0=1*mV

eqs_neurons='''
dv/dt=-v/tau : volt
'''

P=NeuronGroup(10,eqs_neurons,threshold=10*mV,reset=0*mV)
input=PoissonGroup(10,rates=10*Hz)
synapses=Connection(input,P,'v',weight=4*mV)

eqs_stdp='''
x : 1 # fictional presynaptic variable
dc/dt = -c/tau_c : 1 # your postsynaptic calcium variable
v : volt # a copy of the postsynaptic v
'''
stdp=STDP(synapses, eqs=eqs_stdp,pre="w+=w0*(c>1)*(v>5*mV);x",post="c+=1;v", wmax=10*mV)
stdp.post_group.v=linked_var(P,'v')

run(1000*ms)



from base import *
from sparsematrix import *
from connectionvector import *

__all__ = [
         'ConnectionMatrix',
         'SparseConnectionMatrix',
         'DenseConnectionMatrix',
         'DynamicConnectionMatrix',
         ]

class ConnectionMatrix(object):
    '''
    Base class for connection matrix objects
    
    Connection matrix objects support a subset of the following methods:
    
    ``get_row(i)``, ``get_col(i)``
        Returns row/col ``i`` as a :class:`DenseConnectionVector` or
        :class:`SparseConnectionVector` as appropriate for the class.
    ``set_row(i, val)``, ``set_col(i, val)``
        Sets row/col with an array, :class:`DenseConnectionVector` or
        :class:`SparseConnectionVector` (if supported).
    ``get_element(i, j)``, ``set_element(i, j, val)``
        Gets or sets a single value.
    ``get_rows(rows)``
        Returns a list of rows, should be implemented without Python
        function calls for efficiency if possible.
    ``get_cols(cols)``
        Returns a list of cols, should be implemented without Python
        function calls for efficiency if possible.
    ``insert(i,j,x)``, ``remove(i,j)``
        For sparse connection matrices which support it, insert a new
        entry or remove an existing one.
    ``getnnz()``
        Return the number of nonzero entries.
    ``todense()``
        Return the matrix as a dense array.
    
    The ``__getitem__`` and ``__setitem__`` methods are implemented by
    default, and automatically select the appropriate methods from the
    above in the cases where the item to be got or set is of the form
    ``:``, ``i,:``, ``:,j`` or ``i,j``.
    '''
    # methods to be implemented by subclass
    def get_row(self, i):
        return NotImplemented

    def get_col(self, i):
        return NotImplemented

    def set_row(self, i, x):
        return NotImplemented

    def set_col(self, i, x):
        return NotImplemented

    def set_element(self, i, j, x):
        return NotImplemented

    def get_element(self, i, j):
        return NotImplemented

    def get_rows(self, rows):
        return [self.get_row(i) for i in rows]

    def get_cols(self, cols):
        return [self.get_col(i) for i in cols]

    def insert(self, i, j, x):
        return NotImplemented

    def remove(self, i, j):
        return NotImplemented

    def getnnz(self):
        return NotImplemented

    def todense(self):
        return array([todense(r) for r in self])
    # we support the following indexing schemes:
    # - s[:]
    # - s[i,:]
    # - s[:,i]
    # - s[i,j]

    def __getitem__(self, item):
        if isinstance(item, tuple) and isinstance(item[0], int) and item[1] == colon_slice:
            return self.get_row(item[0])
        if isinstance(item, slice):
            if item == colon_slice:
                return self
            else:
                raise ValueError(str(item) + ' not supported.')
        if isinstance(item, int):
            return self.get_row(item)
        if isinstance(item, tuple):
            if len(item) != 2:
                raise TypeError('Only 2D indexing supported.')
            item_i, item_j = item
            if isinstance(item_i, int) and isinstance(item_j, slice):
                if item_j == colon_slice:
                    return self.get_row(item_i)
                raise ValueError('Only ":" indexing supported.')
            if isinstance(item_i, slice) and isinstance(item_j, int):
                if item_i == colon_slice:
                    return self.get_col(item_j)
                raise ValueError('Only ":" indexing supported.')
            if isinstance(item_i, int) and isinstance(item_j, int):
                return self.get_element(item_i, item_j)
            raise TypeError('Only (i,:), (:,j), (i,j) indexing supported.')
        raise TypeError('Can only get items of type slice or tuple')

    def __setitem__(self, item, value):
        if isinstance(item, tuple) and isinstance(item[0], int) and item[1] == colon_slice:
            return self.set_row(item[0], value)
        if isinstance(item, slice):
            raise ValueError(str(item) + ' not supported.')
        if isinstance(item, int):
            return self.set_row(item, value)
        if isinstance(item, tuple):
            if len(item) != 2:
                raise TypeError('Only 2D indexing supported.')
            item_i, item_j = item
            if isinstance(item_i, int) and isinstance(item_j, slice):
                if item_j == colon_slice:
                    return self.set_row(item_i, value)
                raise ValueError('Only ":" indexing supported.')
            if isinstance(item_i, slice) and isinstance(item_j, int):
                if item_i == colon_slice:
                    return self.set_col(item_j, value)
                raise ValueError('Only ":" indexing supported.')
            if isinstance(item_i, int) and isinstance(item_j, int):
                return self.set_element(item_i, item_j, value)
            raise TypeError('Only (i,:), (:,j), (i,j) indexing supported.')
        raise TypeError('Can only set items of type slice or tuple')


class DenseConnectionMatrix(ConnectionMatrix, numpy.ndarray):
    '''
    Dense connection matrix
    
    See documentation for :class:`ConnectionMatrix` for details on
    connection matrix types.

    This matrix implements a dense connection matrix. It is just
    a numpy array. The ``get_row`` and ``get_col`` methods return
    :class:`DenseConnectionVector`` objects.
    '''
    def __new__(subtype, data, **kwds):
        if 'copy' not in kwds:
            kwds = dict(kwds.iteritems())
            kwds['copy'] = False
        return numpy.array(data, **kwds).view(subtype)

    def __init__(self, val, **kwds):
        # precompute rows and cols for fast returns by get_rows etc.
        self.rows = [DenseConnectionVector(numpy.ndarray.__getitem__(self, i)) for i in xrange(val.shape[0])]
        self.cols = [DenseConnectionVector(numpy.ndarray.__getitem__(self, (slice(None), i))) for i in xrange(val.shape[1])]

    def get_rows(self, rows):
        return [self.rows[i] for i in rows]

    def get_cols(self, cols):
        return [self.cols[i] for i in cols]

    def get_row(self, i):
        return self.rows[i]

    def get_col(self, i):
        return self.cols[i]

    def set_row(self, i, x):
        numpy.ndarray.__setitem__(self, i, todense(x))

    def set_col(self, i, x):
        numpy.ndarray.__setitem__(self, (colon_slice, i), todense(x))
        #self[:, i] = todense(x)

    def get_element(self, i, j):
        return numpy.ndarray.__getitem__(self, (i, j))
        #return self[i,j]

    def set_element(self, i, j, val):
        numpy.ndarray.__setitem__(self, (i, j), val)
        #self[i,j] = val
    insert = set_element

    def remove(self, i, j):
        numpy.ndarray.__setitem__(self, (i, j), 0)
        #self[i, j] = 0


class SparseConnectionMatrix(ConnectionMatrix):
    '''
    Sparse connection matrix
    
    See documentation for :class:`ConnectionMatrix` for details on
    connection matrix types.
        
    This class implements a sparse matrix with a fixed number of nonzero
    entries. Row access is very fast, and if the ``column_access`` keyword
    is ``True`` then column access is also supported (but is not as fast
    as row access).
    
    The matrix should be initialised with a scipy sparse matrix.
    
    The ``get_row`` and ``get_col`` methods return
    :class:`SparseConnectionVector` objects. In addition to the
    usual slicing operations supported, ``M[:]=val`` is supported, where
    ``val`` must be a scalar or an array of length ``nnz``.
    
    Implementation details:
    
    The values are stored in an array ``alldata`` of length ``nnz`` (number
    of nonzero entries). The slice ``alldata[rowind[i]:rowind[i+1]]`` gives
    the values for row ``i``. These slices are stored in the list ``rowdata``
    so that ``rowdata[i]`` is the data for row ``i``. The array ``rowj[i]``
    gives the corresponding column ``j`` indices. For row access, the
    memory requirements are 12 bytes per entry (8 bytes for the float value,
    and 4 bytes for the column indices). The array ``allj`` of length ``nnz``
    gives the column ``j`` coordinates for each element in ``alldata`` (the
    elements of ``rowj`` are slices of this array so no extra memory is
    used).
    
    If column access is being used, then in addition to the above there are
    lists ``coli`` and ``coldataindices``. For column ``j``, the array
    ``coli[j]`` gives the row indices for the data values in column ``j``,
    while ``coldataindices[j]`` gives the indices in the array ``alldata``
    for the values in column ``j``. Column access therefore involves a
    copy operation rather than a slice operation. Column access increases
    the memory requirements to 20 bytes per entry (4 extra bytes for the
    row indices and 4 extra bytes for the data indices).
    '''
    def __init__(self, val, column_access=True, **kwds):
        self._useaccel = get_global_preference('useweave')
        self._cpp_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._cpp_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']
        self.nnz = nnz = val.getnnz()# nnz stands for number of nonzero entries
        alldata = numpy.zeros(nnz)
        if column_access:
            colind = numpy.zeros(val.shape[1] + 1, dtype=int)
        allj = numpy.zeros(nnz, dtype=int)
        rowind = numpy.zeros(val.shape[0] + 1, dtype=int)
        rowdata = []
        rowj = []
        if column_access:
            coli = []
            coldataindices = []
        i = 0 # i points to the current index in the alldata array as we go through row by row
        for c in xrange(val.shape[0]):
            # extra the row values and column indices of row c of the initialising matrix
            # this works for any of the scipy sparse matrix formats
            if isinstance(val, sparse.lil_matrix):
                r = val.rows[c]
                d = val.data[c]
            else:
                sr = val[c, :]
                sr = sr.tolil()
                r = sr.rows[0]
                d = sr.data[0]
            # copy the values into the alldata array, the indices into the allj array, and
            # so forth
            rowind[c] = i
            alldata[i:i + len(d)] = d
            allj[i:i + len(r)] = r
            rowdata.append(alldata[i:i + len(d)])
            rowj.append(allj[i:i + len(r)])
            i = i + len(r)
        rowind[val.shape[0]] = i
        if column_access:
            # counts the number of nonzero elements in each column
            counts = zeros(val.shape[1], dtype=int)
            if len(allj):
                bincounts = numpy.bincount(allj)
            else:
                bincounts = numpy.array([], dtype=int)
            counts[:len(bincounts)] = bincounts # ensure that counts is the right length
            # two algorithms depending on whether weave is available
            if self._useaccel:
                # this algorithm just goes through one by one adding each
                # element to the appropriate bin whose sizes we have
                # precomputed. alldi will contain all the data indices
                # in blocks alldi[s[i]:s[i+1]] of length counts[i], and
                # curcdi[i] is the current offset into each block. s is
                # therefore just the cumulative sum of counts.
                curcdi = numpy.zeros(val.shape[1], dtype=int)
                allcoldataindices = numpy.zeros(nnz, dtype=int)
                colind[:] = numpy.hstack(([0], cumsum(counts)))
                colalli = numpy.zeros(nnz, dtype=int)
                numrows = val.shape[0]
                code = '''
                int i = 0;
                for(int k=0;k<nnz;k++)
                {
                    while(k>=rowind[i+1]) i++;
                    int j = allj[k];
                    allcoldataindices[colind[j]+curcdi[j]] = k;
                    colalli[colind[j]+curcdi[j]] = i;
                    curcdi[j]++;
                }
                '''
                weave.inline(code, ['nnz', 'allj', 'allcoldataindices',
                                    'rowind', 'numrows',
                                    'curcdi', 'colind', 'colalli'],
                             compiler=self._cpp_compiler,
                             extra_compile_args=self._extra_compile_args)
                # now store the blocks of allcoldataindices in coldataindices and update coli too
                for i in xrange(len(colind) - 1):
                    D = allcoldataindices[colind[i]:colind[i + 1]]
                    I = colalli[colind[i]:colind[i + 1]]
                    coldataindices.append(D)
                    coli.append(I)
            else:
                # now allj[a] will be the columns in order, so that
                # the first counts[0] elements of allj[a] will be 0,
                # or in other words the first counts[0] elements of a
                # will be the data indices of the elements (i,j) with j==0
                # mergesort is necessary because we want the relative ordering
                # of the elements of a within a block to be maintained
                allcoldataindices = a = argsort(allj, kind='mergesort')
                # this defines colind so that a[colind[i]:colind[i+1]] are the data
                # indices where j==i
                colind[:] = numpy.hstack(([0], cumsum(counts)))
                # this computes the row index of each entry by first generating
                # expanded_row_indices which gives the corresponding row index
                # for each entry enumerated row-by-row, and then using the
                # array allcoldataindices to index this array to convert into
                # the corresponding row index for each entry enumerated
                # col-by-col.
                if len(a):
                    expanded_row_indices = empty(len(a), dtype=int)
                    for k, (i, j) in enumerate(zip(rowind[:-1], rowind[1:])):
                        expanded_row_indices[i:j] = k
                    colalli = expanded_row_indices[a]
                else:
                    colalli = numpy.zeros(nnz, dtype=int)
                # in this loop, I are the data indices where j==i
                # and alli[I} are the corresponding i coordinates
                for i in xrange(len(colind) - 1):
                    D = a[colind[i]:colind[i + 1]]
                    I = colalli[colind[i]:colind[i + 1]]
                    coldataindices.append(D)
                    coli.append(I)

        self.alldata = alldata
        self.rowdata = rowdata
        self.allj = allj
        self.rowj = rowj
        self.rowind = rowind
        self.shape = val.shape
        self.column_access = column_access
        if column_access:
            self.colalli = colalli
            self.coli = coli
            self.coldataindices = coldataindices
            self.allcoldataindices = allcoldataindices
            self.colind = colind
        self.rows = [SparseConnectionVector(self.shape[1], self.rowj[i], self.rowdata[i]) for i in xrange(self.shape[0])]

    def getnnz(self):
        return self.nnz

    def get_element(self, i, j):
        n = searchsorted(self.rowj[i], j)
        if n >= len(self.rowj[i]) or self.rowj[i][n] != j:
            return 0
        return self.rowdata[i][n]

    def set_element(self, i, j, x):
        n = searchsorted(self.rowj[i], j)
        if n >= len(self.rowj[i]) or self.rowj[i][n] != j:
            raise ValueError('Insertion of new elements not supported for SparseConnectionMatrix.')
        self.rowdata[i][n] = x

    def get_row(self, i):
        return self.rows[i]

    def get_rows(self, rows):
        return [self.rows[i] for i in rows]

    def get_col(self, j):
        if self.column_access:
            return SparseConnectionVector(self.shape[0], self.coli[j], self.alldata[self.coldataindices[j]])
        else:
            raise TypeError('No column access.')

    def get_cols(self, cols):
        if self.column_access:
            return [SparseConnectionVector(self.shape[0], self.coli[j], self.alldata[self.coldataindices[j]]) for j in cols]
        else:
            raise TypeError('No column access.')

    def set_row(self, i, val):
        if isinstance(val, SparseConnectionVector):
            if val.ind is not self.rowj[i]:
                if not (val.ind == self.rowj[i]).all():
                    raise ValueError('Sparse row setting must use same indices.')
            self.rowdata[i][:] = val
        else:
            if isinstance(val, numpy.ndarray):
                val = asarray(val)
                self.rowdata[i][:] = val[self.rowj[i]]
            else:
                self.rowdata[i][:] = val

    def set_col(self, j, val):
        if self.column_access:
            if isinstance(val, SparseConnectionVector):
                if val.ind is not self.coli[j]:
                    if not (val.ind == self.coli[j]).all():
                        raise ValueError('Sparse col setting must use same indices.')
                self.alldata[self.coldataindices[j]] = val
            else:
                if isinstance(val, numpy.ndarray):
                    val = asarray(val)
                    self.alldata[self.coldataindices[j]] = val[self.coli[j]]
                else:
                    self.alldata[self.coldataindices[j]] = val
        else:
            raise TypeError('No column access.')

    def __setitem__(self, item, value):
        if item == colon_slice:
            self.alldata[:] = value
        else:
            ConnectionMatrix.__setitem__(self, item, value)


class DynamicConnectionMatrix(ConnectionMatrix):
    '''
    Dynamic (sparse) connection matrix
    
    See documentation for :class:`ConnectionMatrix` for details on
    connection matrix types.
        
    This class implements a sparse matrix with a variable number of nonzero
    entries. Row access and column access are provided, but are not as fast
    as for :class:`SparseConnectionMatrix`.
    
    The matrix should be initialised with a scipy sparse matrix.
    
    The ``get_row`` and ``get_col`` methods return
    :class:`SparseConnectionVector` objects. In addition to the
    usual slicing operations supported, ``M[:]=val`` is supported, where
    ``val`` must be a scalar or an array of length ``nnz``.
    
    **Implementation details**
    
    The values are stored in an array ``alldata`` of length ``nnzmax`` (maximum
    number of nonzero entries). This is a dynamic array, see:
    
        http://en.wikipedia.org/wiki/Dynamic_array
        
    You can set the resizing constant with the argument ``dynamic_array_const``.
    Normally the default value 2 is fine but if memory is a worry it could be
    made smaller.
    
    Rows and column point in to this data array, and the list ``rowj`` consists
    of an array of column indices for each row, with ``coli`` containing arrays
    of row indices for each column. Similarly, ``rowdataind`` and ``coldataind``
    consist of arrays of pointers to the indices in the ``alldata`` array. 
    '''
    def __init__(self, val, nnzmax=None, dynamic_array_const=2, **kwds):
        self.shape = val.shape
        self.dynamic_array_const = dynamic_array_const
        if nnzmax is None or nnzmax < val.getnnz():
            nnzmax = val.getnnz()
        self.nnzmax = nnzmax
        self.nnz = val.getnnz()
        self.alldata = numpy.zeros(nnzmax)
        self.unusedinds = range(self.nnz, self.nnzmax)
        i = 0
        self.rowj = []
        self.rowdataind = []
        alli = zeros(self.nnz, dtype=int)
        allj = zeros(self.nnz, dtype=int)
        for c in xrange(val.shape[0]):
            # extra the row values and column indices of row c of the initialising matrix
            # this works for any of the scipy sparse matrix formats
            if isinstance(val, sparse.lil_matrix):
                r = val.rows[c]
                d = val.data[c]
            else:
                sr = val[c, :]
                sr = sr.tolil()
                r = sr.rows[0]
                d = sr.data[0]
            self.alldata[i:i + len(d)] = d
            self.rowj.append(array(r, dtype=int))
            self.rowdataind.append(arange(i, i + len(d)))
            allj[i:i + len(d)] = r
            alli[i:i + len(d)] = c
            i += len(d)
        # now update the coli and coldataind variables
        self.coli = []
        self.coldataind = []
        # counts the number of nonzero elements in each column
        if numpy.__version__ >= '1.3.0':
            counts = numpy.histogram(allj, numpy.arange(val.shape[1] + 1, dtype=int))[0]
        else:
            counts = numpy.histogram(allj, numpy.arange(val.shape[1] + 1, dtype=int), new=True)[0]
        # now we have to go through one by one unfortunately, and so we keep curcdi, the
        # current column data index for each column
        curcdi = numpy.zeros(val.shape[1], dtype=int)
        # initialise the memory for the column data indices
        for j in xrange(val.shape[1]):
            self.coldataind.append(numpy.zeros(counts[j], dtype=int))
        # one by one for every element, update the dataindices and curcdi data pointers
        for i, j in enumerate(allj):
            self.coldataind[j][curcdi[j]] = i
            curcdi[j] += 1
        for j in xrange(val.shape[1]):
            self.coli.append(alli[self.coldataind[j]])

    def getnnz(self):
        return self.nnz

    def insert(self, i, j, x):
        n = searchsorted(self.rowj[i], j)
        if n < len(self.rowj[i]) and self.rowj[i][n] == j:
            self.alldata[self.rowdataind[i][n]] = x
            return
        m = searchsorted(self.coli[j], i)
        if self.nnz == self.nnzmax:
            # reallocate memory using a dynamic array structure (amortized O(1) cost for append)
            newnnzmax = int(self.nnzmax * self.dynamic_array_const)
            if newnnzmax <= self.nnzmax:
                newnnzmax += 1
            if newnnzmax > self.shape[0] * self.shape[1]:
                newnnzmax = self.shape[0] * self.shape[1]
            self.alldata = hstack((self.alldata, numpy.zeros(newnnzmax - self.nnzmax, dtype=self.alldata.dtype)))
            self.unusedinds.extend(range(self.nnz, newnnzmax))
            self.nnzmax = newnnzmax
        newind = self.unusedinds.pop(-1)
        self.alldata[newind] = x
        self.nnz += 1
        # update row
        newrowj = numpy.zeros(len(self.rowj[i]) + 1, dtype=int)
        newrowj[:n] = self.rowj[i][:n]
        newrowj[n] = j
        newrowj[n + 1:] = self.rowj[i][n:]
        self.rowj[i] = newrowj
        newrowdataind = numpy.zeros(len(self.rowdataind[i]) + 1, dtype=int)
        newrowdataind[:n] = self.rowdataind[i][:n]
        newrowdataind[n] = newind
        newrowdataind[n + 1:] = self.rowdataind[i][n:]
        self.rowdataind[i] = newrowdataind
        # update col
        newcoli = numpy.zeros(len(self.coli[j]) + 1, dtype=int)
        newcoli[:m] = self.coli[j][:m]
        newcoli[m] = i
        newcoli[m + 1:] = self.coli[j][m:]
        self.coli[j] = newcoli
        newcoldataind = numpy.zeros(len(self.coldataind[j]) + 1, dtype=int)
        newcoldataind[:m] = self.coldataind[j][:m]
        newcoldataind[m] = newind
        newcoldataind[m + 1:] = self.coldataind[j][m:]
        self.coldataind[j] = newcoldataind

    def remove(self, i, j):
        n = searchsorted(self.rowj[i], j)
        if n >= len(self.rowj[i]) or self.rowj[i][n] != j:
            raise ValueError('No element to remove at position ' + str(i, j))
        oldind = self.rowdataind[i][n]
        self.unusedinds.append(oldind)
        self.nnz -= 1
        m = searchsorted(self.coli[j], i)
        # update row
        newrowj = numpy.zeros(len(self.rowj[i]) - 1, dtype=int)
        newrowj[:n] = self.rowj[i][:n]
        newrowj[n:] = self.rowj[i][n + 1:]
        self.rowj[i] = newrowj
        newrowdataind = numpy.zeros(len(self.rowdataind[i]) - 1, dtype=int)
        newrowdataind[:n] = self.rowdataind[i][:n]
        newrowdataind[n:] = self.rowdataind[i][n + 1:]
        self.rowdataind[i] = newrowdataind
        # update col
        newcoli = numpy.zeros(len(self.coli[j]) - 1, dtype=int)
        newcoli[:m] = self.coli[j][:m]
        newcoli[m:] = self.coli[j][m + 1:]
        self.coli[j] = newcoli
        newcoldataind = numpy.zeros(len(self.coldataind[j]) - 1, dtype=int)
        newcoldataind[:m] = self.coldataind[j][:m]
        newcoldataind[m:] = self.coldataind[j][m + 1:]
        self.coldataind[j] = newcoldataind

    def get_element(self, i, j):
        n = searchsorted(self.rowj[i], j)
        if n >= len(self.rowj[i]) or self.rowj[i][n] != j:
            return 0
        return self.alldata[self.rowdataind[i][n]]

    set_element = insert

    def get_row(self, i):
        return SparseConnectionVector(self.shape[1], self.rowj[i], self.alldata[self.rowdataind[i]])

    def get_rows(self, rows):
        return [SparseConnectionVector(self.shape[1], self.rowj[i], self.alldata[self.rowdataind[i]]) for i in rows]

    def get_col(self, j):
        return SparseConnectionVector(self.shape[0], self.coli[j], self.alldata[self.coldataind[j]])

    def get_cols(self, cols):
        return [SparseConnectionVector(self.shape[0], self.coli[j], self.alldata[self.coldataind[j]]) for j in cols]

    def set_row(self, i, val):
        if isinstance(val, SparseConnectionVector):
            if val.ind is not self.rowj[i]:
                if not (val.ind == self.rowj[i]).all():
                    raise ValueError('Sparse row setting must use same indices.')
            self.alldata[self.rowdataind[i]] = val
        else:
            if isinstance(val, numpy.ndarray):
                val = asarray(val)
                self.alldata[self.rowdataind[i]] = val[self.rowj[i]]
            else:
                self.alldata[self.rowdataind[i]] = val

    def set_col(self, j, val):
        if isinstance(val, SparseConnectionVector):
            if val.ind is not self.coli[j]:
                if not (val.ind == self.coli[j]).all():
                    raise ValueError('Sparse row setting must use same indices.')
            self.alldata[self.coldataind[j]] = val
        else:
            if isinstance(val, numpy.ndarray):
                val = asarray(val)
                self.alldata[self.coldataind[j]] = val[self.coli[j]]
            else:
                self.alldata[self.coldataind[j]] = val

    def __setitem__(self, item, value):
        if item == colon_slice:
            self.alldata[:self.nnz] = value
        else:
            ConnectionMatrix.__setitem__(self, item, value)



from brian import *
import numpy

__all__ = ['RefractoryBeforeThresholdNeuronGroup', 'NeuronGroup']


class RefractoryBeforeThresholdNeuronGroup(NeuronGroup):
    '''
    NeuronGroup that performs refractoriness before thresholding
    
    If you have neurons that receiving very strong inputs enough to drive them
    from reset to threshold in a single time step dt then you can use this
    temporary hacked version of NeuronGroup to prevent spiking during the
    refractory period. Performance may not be great, and possibly the refractory
    period will be what you have given + dt. This should be fixed in a
    later release of Brian.
    '''
    def update(self):
        self._state_updater(self) # update the variables
        if isinstance(self._resetfun, Refractoriness):
            self._resetfun(self)
        spikes = self._threshold(self) # get spikes
        if not isinstance(spikes, numpy.ndarray):
            spikes = array(spikes, dtype=int)
        self.LS.push(spikes) # Store spikes

NeuronGroup = RefractoryBeforeThresholdNeuronGroup

if __name__ == '__main__':
    #from brian import NeuronGroup
    G = NeuronGroup(1, 'V:1', threshold=1, reset=0, refractory=5 * ms)
    @network_operation(when='before_groups')
    def f():
        G.V = 2
    M = StateMonitor(G, 'V', record=True, when='before_resets')
    sp = SpikeMonitor(G)
    run(40 * ms)
    print sp.spikes
    plot(M.times, M[0])
    show()



from brian import *
import brianlib as bl
import time
####### Parameters ############################
domonitorandplot = False
debugmode = False
duration = 2.5 * second
we = 1.62 * mV # 1.62*mV for low firing rate, 9*mV for high
N = 100
####### Definition of CUBA network ############
Nsyn = 20.
eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''
Ne = int(N * 0.8)
Ni = N - Ne
P = NeuronGroup(N, model=eqs,
              threshold= -50 * mV, reset= -60 * mV, refractory=5 * ms)
Pe = P.subgroup(Ne)
Pi = P.subgroup(Ni)
if domonitorandplot:
    Pe.spikemonitor = SpikeMonitor(Pe)
    Pi.spikemonitor = SpikeMonitor(Pi)
    P.spikemonitor = SpikeMonitor(P)
Ce = Connection(Pe, P, 'ge', structure='dense')
Ci = Connection(Pi, P, 'gi', structure='dense')
cp = Nsyn / N
Ce.connect_random(Pe, P, cp, weight=we)
Ci.connect_random(Pi, P, cp, weight= -9 * mV)
if domonitorandplot:
    M = StateMonitor(P, 'v', record=True)
P.v = -60 * mV + 10 * mV * rand(len(P))
P.ge = we * rand(len(P))
P.gi = -9 * mV * rand(len(P))
####### Generate cBrian version ###############
c = array(P._state_updater._C.flatten()) # if we don't do this the memory is corrupted
blPsu = bl.LinearStateUpdater(P._state_updater.A, c)
if debugmode:
    print 'cBrian state updater initialised OK'
blPthr = bl.Threshold(P._threshold.state, float(P._threshold.threshold))
if debugmode:
    print 'cBrian threshold initialised OK'
period = int(P._resetfun.period / P.clock.dt) + 1
blPreset = bl.Refractoriness(P._resetfun.state, float(P._resetfun.resetvalue), period)
if debugmode:
    print 'cBrian reset initialised OK'
blP = bl.NeuronGroup(P._S, blPsu, blPthr, blPreset, P.LS.S.n, P.LS.ind.n)
if debugmode:
    print 'cBrian P initialised OK'
blPe = bl.NeuronGroup(blP, Pe._origin, len(Pe))
if debugmode:
    print 'cBrian Pe initialised OK'
blPi = bl.NeuronGroup(blP, Pi._origin, len(Pi))
if debugmode:
    print 'cBrian Pi initialised OK'
blCe_W = bl.DenseConnectionMatrix(Ce.W)
if debugmode:
    print 'cBrian Ce_W initialised OK'
blCi_W = bl.DenseConnectionMatrix(Ci.W)
if debugmode:
    print 'cBrian Ci_w initialised OK'
blCe = bl.Connection(blPe, blP, blCe_W, P.get_var_index('ge'))
if debugmode:
    print 'cBrian Ce initialised OK'
blCi = bl.Connection(blPi, blP, blCi_W, P.get_var_index('gi'))
if debugmode:
    print 'cBrian Ci initialised OK'
if domonitorandplot:
    blM = bl.StateMonitor(blP, 0)
    if debugmode:
        print 'cBrian M initialised OK'
blnet = bl.Network()
if debugmode:
    print 'cBrian net initialised OK'
blnet.add(blP)
if debugmode:
    print 'cBrian P added to net OK'
blnet.add(blCe)
if debugmode:
    print 'cBrian Ce added to net OK'
blnet.add(blCi)
if debugmode:
    print 'cBrian Ci added to net OK'
if domonitorandplot:
    blnet.add(blM)
    if debugmode:
        print 'cBrian M added to net OK'
####### Test cBrian in debugmode ##############
#if debugmode:
#    blP.update()
#    print 'debugmode cBrian blP.update() OK'
#    blP.reset()
#    print 'debugmode cBrian blP.reset() OK'
####### Copy data #############################
V_start = array(P.v, copy=True)
ge_start = array(P.ge, copy=True)
gi_start = array(P.gi, copy=True)
####### Run cBrian version ####################
P.v = V_start
P.ge = ge_start
P.gi = gi_start
start = time.time()
if domonitorandplot:
    blsm_P = []
    blsm_Pe = []
    blsm_Pi = []
    from itertools import repeat
    for t in [i * defaultclock.dt for i in range(int(duration / defaultclock.dt))]:
        blnet.update()
        blsm_P.extend(zip(blP.get_spikes(), repeat(t)))
        blsm_Pe.extend(zip(blPe.get_spikes(), repeat(t)))
        blsm_Pi.extend(zip(blPi.get_spikes(), repeat(t)))
else:
    blnet.run(int(duration / defaultclock.dt))
print 'cBrian:', (time.time() - start) * second
V_after_cBrian = array(P.v, copy=True)
####### Run Brian version #####################
P.v = V_start
P.ge = ge_start
P.gi = gi_start
start = time.time()
run(duration)
print 'Brian:', (time.time() - start) * second
V_after_Brian = array(P.v, copy=True)
####### Compare ###############################
print 'max abs diff of V values at end of sim:', max(abs(V_after_Brian - V_after_cBrian))
if domonitorandplot:
    subplot(211)
    for i in range(10):
        plot(blM[i])
    title('cBrian')
    subplot(212)
    for i in range(10):
        plot(M[i])
    title('Brian')
    figure()
    subplot(311)
    raster_plot(P.spikemonitor)
    subplot(312)
    raster_plot(Pe.spikemonitor)
    subplot(313)
    raster_plot(Pi.spikemonitor)
    subplot(311)
    if len(blsm_P):
        i, t = zip(*blsm_P)
        plot(array(t) / ms, i, '+')
    subplot(312)
    if len(blsm_Pe):
        i, t = zip(*blsm_Pe)
        plot(array(t) / ms, i, '+')
    subplot(313)
    if len(blsm_Pi):
        i, t = zip(*blsm_Pi)
        plot(array(t) / ms, i, '+')
    show()



'''
Compartmental neurons
See BEP-15

TODO:
* Threshold and reset are special (not as normal NeuronGroup because only 1 spike)
* Hines method
* Point processes
* StateMonitor
* neuron.plot('gl')
* Iteration (over the branch or the entire tree?)
'''
from morphology import *
from brian.stdunits import *
from brian.units import *
from brian.reset import NoReset
from brian.stateupdater import StateUpdater
from brian.equations import Equations
from brian.group import Group
from itertools import count
from brian.neurongroup import NeuronGroup

__all__ = ['SpatialNeuron', 'CompartmentalNeuron']


class SpatialNeuron(NeuronGroup):
    """
    Compartmental model with morphology.
    """
    def __init__(self, morphology=None, model=None, threshold=None, reset=NoReset(),
                 refractory=0 * ms, level=0,
                 clock=None, unit_checking=True,
                 compile=False, freeze=False, cm=0.9 * uF / cm ** 2, Ri=150 * ohm * cm):
        clock = guess_clock(clock)
        N = len(morphology) # number of compartments

        # Equations for morphology
        eqs_morphology = Equations("""
        diameter : um
        length : um
        x : um
        y : um
        z : um
        area : um**2
        """)

        # Create the state updater
        if isinstance(model, str):
            model = Equations(model, level=level + 1)
        model += Equations('''
        v:volt # membrane potential
        #Im:amp/cm**2 # membrane current (should we have it?)
        ''')
        full_model = model + eqs_morphology
        Group.__init__(self, full_model, N, unit_checking=unit_checking)
        self._eqs = model
        self._state_updater = SpatialStateUpdater(model, clock)
        var_names = full_model._diffeq_names
        self.cm = cm # could be a vector?
        self.Ri = Ri
        S0 = {}
        # Fill missing units
        for key, value in full_model._units.iteritems():
            if not key in S0:
                S0[key] = 0 * value
        self._S0 = [0] * len(var_names)
        for var, i in zip(var_names, count()):
            self._S0[i] = S0[var]

        NeuronGroup.__init__(self, N, model=self._state_updater, threshold=threshold, reset=reset, refractory=refractory,
                             level=level + 1, clock=clock, unit_checking=unit_checking)

        # Insert morphology
        self.morphology = morphology
        self.morphology.compress(diameter=self.diameter, length=self.length, x=self.x, y=self.y, z=self.z, area=self.area)

    def subgroup(self, N): # Subgrouping cannot be done in this way
        raise NotImplementedError

    def __getitem__(self, x):
        '''
        Subgrouping mechanism.
        self['axon'] returns the subtree named "axon".
        
        TODO:
        self[:] returns the full branch.
        '''
        morpho = self.morphology[x]
        N = self[morpho._origin:morpho._origin + len(morpho)]
        N.morphology = morpho
        return N

    def __getattr__(self, x):
        if (x != 'morphology') and ((x in self.morphology._namedkid) or all([c in 'LR123456789' for c in x])): # subtree
            return self[x]
        else:
            return NeuronGroup.__getattr__(self, x)


class SpatialStateUpdater(StateUpdater):
    """
    State updater for compartmental models.
    """
    def __init__(self, eqs, clock=None):
        self.eqs = eqs

    def __len__(self):
        '''
        Number of state variables
        '''
        return len(self.eqs)

CompartmentalNeuron = SpatialNeuron

if __name__ == '__main__':
    from brian import *
    morpho = Morphology('oi24rpy1.CNG.swc') # visual L3 pyramidal cell
    print len(morpho), "compartments"
    El = -70 * mV
    eqs = ''' # The same equations for the whole neuron, but possibly different parameter values
    Im=gl*(El-v) : amp/cm**2 # distributed transmembrane current
    gl : siemens/cm**2 # spatially distributed conductance
    '''
    neuron = SpatialNeuron(morphology=morpho, threshold="axon[50*um].v>0*mV", model=eqs, refractory=4 * ms, cm=0.9 * uF / cm ** 2, Ri=150 * ohm * cm)
    neuron.axon[0 * um:50 * um].gl = 1e-3 * siemens / cm ** 2
    print sum(neuron.axon.gl)
    print neuron.axon[40 * um].gl
    #branch=neuron.axon[0*um:50*um]
    neuron.morphology.plot()
    show()



'''
Transient synchronisation in a population of noisy IF neurons
with distance-dependent synaptic weights (organised as a ring)
'''
from brian import *

tau = 10 * ms
N = 100
v0 = 5 * mV
sigma = 4 * mV
group = NeuronGroup(N, model='dv/dt=(v0-v)/tau + sigma*xi/tau**.5 : volt', \
                  threshold=10 * mV, reset=0 * mV)
C = Connection(group, group, 'v', weight=lambda i, j:.4 * mV * cos(2. * pi * (i - j) * 1. / N))
S = SpikeMonitor(group)
R = PopulationRateMonitor(group)
group.v = rand(N) * 10 * mV

run(5000 * ms)
subplot(211)
raster_plot(S)
subplot(223)
imshow(C.W.todense(), interpolation='nearest')
title('Synaptic connections')
subplot(224)
plot(R.times / ms, R.smooth_rate(2 * ms, filter='flat'))
title('Firing rate')
show()



'''
An example of single-electrode current clamp recording
with bridge compensation (using the electrophysiology library).
'''
from brian import *
from brian.library.electrophysiology import *

taum = 20 * ms        # membrane time constant
gl = 1. / (50 * Mohm)   # leak conductance
Cm = taum * gl        # membrane capacitance
Re = 50 * Mohm        # electrode resistance
Ce = 0.5 * ms / Re      # electrode capacitance

eqs = Equations('''
dvm/dt=(-gl*vm+i_inj)/Cm : volt
Rbridge:ohm # bridge resistance
I:amp # command current
''')
eqs += current_clamp(i_cmd='I', Re=Re, Ce=Ce, bridge='Rbridge')
setup = NeuronGroup(1, model=eqs)
soma = StateMonitor(setup, 'vm', record=True)
recording = StateMonitor(setup, 'v_rec', record=True)

# No compensation
run(50 * ms)
setup.I = .5 * nA
run(100 * ms)
setup.I = 0 * nA
run(50 * ms)

# Full compensation
setup.Rbridge = Re
run(50 * ms)
setup.I = .5 * nA
run(100 * ms)
setup.I = 0 * nA
run(50 * ms)

plot(recording.times / ms, recording[0] / mV, 'b')
plot(soma.times / ms, soma[0] / mV, 'r')
show()



from brian import *
import gc
piecesize = 200
def break_up_connection(source, target, var, weight, sparseness, piecesize=piecesize):
    N = len(source)
    pieces = range(0, N, piecesize)+[N]
    conns = []
    for start, end in zip(pieces[:-1], pieces[1:]):
        C = Connection(source[start:end], target, var, weight=weight,
                       sparseness=sparseness,
                       column_access=True)
#        C.compress()
        gc.collect()
        conns.append(C)
#    magic_register(*conns)
    return conns


eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''
P = NeuronGroup(1000, eqs, threshold=-50*mV, reset=-60*mV)
P.v = -60*mV
Pe = P.subgroup(800)
Pi = P.subgroup(200)
Ce = break_up_connection(Pe, P, 'ge', weight=1.62*mV, sparseness=0.2)
Ci = break_up_connection(Pi, P, 'gi', weight=-9*mV, sparseness=0.2)

# Plasticity
gmax = 0.01
tau_pre = 20 * ms
tau_post = tau_pre
dA_pre = .01
dA_post = -dA_pre * tau_pre / tau_post * 2.5
stdp = ExponentialSTDP(Ce[0], tau_pre, tau_post, dA_pre, dA_post, wmax=gmax, update='mixed')

M = SpikeMonitor(P)

run(1*second,report='text')
raster_plot(M)
show()



'''
Neuronal morphology module for Brian.
Old version - maybe a few functions could still be useful (e.g. simplify)

Typical values for intrinsic parameters

Cm =0.7 - 1 uF/(cm**2) # specific membrane capacitance
Ri = 70 - 200 ohm*cm # intracellular resistivity
Rm = variable, choose taum=Rm*Cm first # specific membrane resistance (ohm*cm**2)

TODO:
* discretise
* problem with simplify and location
* check if surfacic when applying on a branch
'''
from brian.units import meter, ohm
from brian.stdunits import um, cm, ms, uF, mV
from numpy import sqrt, array, pi, mean
from brian.compartments import *
from brian.membrane_equations import *
from brian import sum

def space_constant(d, Rm, Ri):
    return (d * Rm / Ri) ** .5


class Morphology(object):
    '''
    Morphology class
    ----------------
    
    '''
    def __init__(self, name=None, Cm=1 * uF / (cm ** 2)):
        '''
        Cm is the specific membrane capacitance
        '''
        self._segments = []
        if name is not None:
            self.load(name, Cm)

    def load(self, name, Cm=1 * uF / (cm ** 2)):
        '''
        Reads a SWC file containing a neuronal morphology and returns
        a morphology object
        Cm is the specific membrane capacitance.
        
        segments is a dictionary of segments, each segment is a dictionary with keys:
        * parent: index of the parent segment
        * type: type of segment in 'undefined','soma','dendrite','axon','apical','fork','end','custom'
        * length
        * area
        * radius
        * location: (x,y,z) coordinates
        
        Large database at http://neuromorpho.org/neuroMorpho
        
        Information below from http://www.mssm.edu/cnic/swc.html
        
        SWC File Format
    
        The format of an SWC file is fairly simple. It is a text file consisting of a header with various fields beginning with a # character, and a series of three dimensional points containing an index, radius, type, and connectivity information. The lines in the text file representing points have the following layout. 
        n T x y z R P
    
        n is an integer label that identifies the current point and increments by one from one line to the next.
        
        T is an integer representing the type of neuronal segment, such as soma, axon, apical dendrite, etc. The standard accepted integer values are given below.
        
            * 0 = undefined
            * 1 = soma
            * 2 = axon
            * 3 = dendrite
            * 4 = apical dendrite
            * 5 = fork point
            * 6 = end point
            * 7 = custom
        
        x, y, z gives the cartesian coordinates of each node.
        
        R is the radius at that node.
        P indicates the parent (the integer label) of the current point or -1 to indicate an origin (soma). 
        '''
        # First create the list of segments
        lines = open(name).read().splitlines()
        segments = []
        branching_points = []
        types = ['undefined', 'soma', 'axon', 'dendrite', 'apical', 'fork', 'end', 'custom']
        for line in lines:
            if line[0] != '#': # comment
                numbers = line.split()
                n = int(numbers[0]) - 1
                T = types[int(numbers[1])]
                x = float(numbers[2]) * um
                y = float(numbers[3]) * um
                z = float(numbers[4]) * um
                R = float(numbers[5]) * um
                P = int(numbers[6]) - 1 # 0-based indexing
                segment = dict(n=n, type=T, location=(x, y, z), radius=R, parent=P)
                if T == 'soma':
                    segment['area'] = 4 * pi * segment['radius'] ** 2
                else: # dendrite
                    segment['length'] = (sum((array(segment['location']) - array(segments[P]['location'])) ** 2)) ** .5 * meter
                    segment['area'] = segment['length'] * 2 * pi * segment['radius']

                if (P != n - 1) and (P > -2): # P is a branching point
                    branching_points.append(P)

                segments.append(segment)

        # Create branches
        # branches = list of dict(begin,end,segments) where segments are segment indexes
        branches = []
        branch = dict(start=0, segments=[], children=0)
        for segment in segments:
            n = segment['n']
            if segment['n'] in branching_points: # end of branch
                branch['segments'].append(n)
                branch['end'] = n
                branches.append(branch)
                branch = dict(start=n + 1, segments=[], children=0)
            elif segment['parent'] != n - 1: # new branch
                branch['end'] = n - 1
                branches.append(branch)
                branch = dict(start=n, segments=[n], children=0)
            else:
                branch['segments'].append(n)
        # Last branch
        branch['end'] = n
        branches.append(branch)

        # Make segment dictionary
        self._segments = dict()
        for segment in segments:
            self._segments[segment['n']] = segment

        # Name branches and segments
        # The soma is 'soma'
        self._branches = dict()
        for branch in branches:
            #if branch['type']
            parent = self._segments[branch['start']]['parent']
            if parent in self._segments:
                b = [b for b in branches if parent in b['segments']][0] # parent branch
                if b['name'] == 'soma':
                    branch['name'] = str(b['children'])
                else:
                    branch['name'] = b['name'] + str(b['children'])
                b['children'] += 1
            else:
                branch['name'] = 'soma'
            self._branches[branch['name']] = branch

        self.build_equations(Cm)

    def info(self):
        '''
        Prints information about the morphology
        '''
        print len([s for n, s in self._segments.iteritems() if s['n'] == n]), 'segments'
        print 'Branches:'
        for name, branch in self._branches.iteritems():
            area = sum([self._segments[s]['area'] for s in branch['segments']]) * meter ** 2
            length = sum([self._segments[s]['length'] for s in branch['segments']\
                        if 'length' in self._segments[s]]) * meter
            print name, ':', len(branch['segments']), 'segments ; area =', area, '; length =', length

    def build_equations(self, Cm=1 * uF / (cm ** 2)):
        '''
        Builds a dictionary of equations.
        Cm is the specific capacitance.
        '''
        self._eqs = dict()
        for n, segment in self._segments.iteritems():
            if segment['n'] == n: # otherwise it is a copy
                self._eqs[n] = MembraneEquation(C=Cm * segment['area'])
                self._eqs[n].area = segment['area']

    def __getitem__(self, x):
        '''
        Returns a branch or a compartment (equations).
        '''
        if type(x) == type((0, 0)): # tuple: compartment
            branch, location = x
            return self.compartment(branch, location)
        else: # single variable: branch
            return self.branch(x)

    def index(self, branch, location):
        '''
        Returns the index of the segment at given location on given branch
        '''
        branch = str(branch)
        branch = self._branches[branch]
        if type(location) == type(0): # index
            return branch['start'] + location
        else: # position
            x = 0 * meter
            oldx = x
            for s in branch['segments']:
                x += self._segments[s]['length']
                if x > location:
                    # The closer one
                    if x - location < location - oldx:
                        return s
                    else:
                        return s - 1
                oldx = x
            raise IndexError, 'Location not found'

    def radius(self, branch, location=None):
        '''
        Returns the radius of a compartment or branch
        '''
        if location is None: # branch: return mean radius
            return mean([self.radius(branch, n) for n in self._branches[branch]['segments']]) * meter
        else:
            seg = self._segments[self.index(branch, location)]
            return seg['radius']

    def diameter(self, branch, location=None):
        '''
        Returns the diameter of a compartment or branch
        '''
        return 2 * self.radius(branch, location)

    def length(self, branch, location=None):
        '''
        Returns the length of a compartment or branch.
        '''
        if location is None: # branch
            l = 0 * meter
            for n in self._branches[branch]['segments']:
                l += self.length(branch, n)
            return l
        else:
            return self._segments[self.index(branch, location)]['length']

    def area(self, branch, location=None):
        '''
        Returns the area of a compartment or branch.
        '''
        if location is None: # branch
            a = 0 * meter ** 2
            for n in self._branches[branch]['segments']:
                a += self.area(branch, n)
            return a
        else:
            seg = self._segments[self.index(branch, location)]
            if seg['type'] == 'soma':
                return 4 * pi * self.radius(branch, location) ** 2
            else:
                return self.length(branch, location) * 2 * pi * self.radius(branch, location)

    def branch(self, branch, children=False):
        '''
        Returns a branch.
        
        children = True: children are also returned (subtree)
        '''
        branch = str(branch)
        if children:
            l = []
            for br in self._branches.iterkeys():
                if (branch == 'soma') or br.startswith(branch):
                    l.extend(self.branch(br))
            return l
        else:
            return [self._eqs[n] for n in self._branches[branch]['segments']]

    def subtree(self, branch):
        return self.branch(branch, children=True)

    def compartment(self, branch, location):
        '''
        Returns the segment at given location on given branch
        '''
        return self._eqs[self.index(branch, location)]

    def equations(self, Ri=100 * ohm * cm):
        '''
        Returns an Equations object for the whole morphology.
        Ri is the intracellular resistivity.
        Connects all segments together.
        '''
        eqs = Compartments(self._eqs)
        for n, s in self._segments.iteritems():
            if s['n'] == n: # otherwise it is a copy
                n0 = s['parent']
                if n0 in self._segments:
                    s0 = self._segments[n0]
                    if s0['type'] == 'soma':
                        l = .5 * s['length']
                    else:
                        l = .5 * (s['length'] + s0['length']) # connection length
                    Ra = Ri * l / (pi * .5 * (s0['radius'] ** 2 + s['radius'] ** 2))
                    eqs.connect(n0, n, Ra)
        return eqs

    def simplify(self):
        '''
        Replaces branches by equivalent cylinders.
        Preserves the total area and electrotonic length.
        '''
        for branch in self._branches.itervalues():
            if branch['name'] != 'soma':
                segs = [self._segments[n] for n in branch['segments']]
                a = sum([seg['area'] for seg in segs])
                l = sum([seg['length'] / sqrt(seg['radius']) for seg in segs])
                r = (a / (2 * pi * l)) ** (2. / 3.)
                l0 = (r ** .5) * l
                segs[0]['radius'] = r * meter
                #segs[0]['length']=l0 # length is the length of connecting segment
                segs[0]['area'] = a * meter ** 2
                self._segments[branch['end']] = segs[0]
                branch['segments'] = branch['segments'][0:1]
                # Destroy other segments
                for seg in segs[1:-1]:
                    del self._segments[seg['n']]

    def insert_current(self, current, branch, location=None):
        if location is None: # on branch
            if current._prefix == '__current_': # point current
                raise ValueError, "The current should be surfacic"
            branch = morpho.branch(branch)
            for c in branch:
                c += current
        else:
            c = morpho.compartment(branch, location)
            c += current

if __name__ == '__main__':
    morpho = Morphology('mp_ma_40984_gc2.CNG.swc') # retinal ganglion cell
    morpho.info()
    #print morpho.branch(100)
    #morpho.simplify()
    #morpho.info()
    morpho.insert_current(Current('I=g*(E-vm) : amp/(cm**2)', surfacic=True,
                                  g=1 * uF / (cm ** 2) / (20 * ms), E= -75 * mV), 101, 10 * um)
    print morpho[101, 10 * um]
    model = morpho.equations(Ri=70 * ohm * cm)



from brian import *
import struct

x = array(rand(10)*1000, dtype = int16)
print 'x',x

x_n = x.newbyteorder('>')
print 'x_n',x_n

s_struct = ''
for v in x_n:
    s_struct += struct.pack('>i', v)   

s_np =x_n.tostring() 

if s_np == s_struct:
    print 'yay'
else:
    print s_np
    print s_struct



xre=fromstring(s_struct, dtype=int16) # or uint16?
xre = xre[:,0].newbyteorder('>')
print 'xre', xre

if (x == xre).all():
    print 'YAY'












from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *
from plot_count import ircam_plot_count
import time
import pickle

hrtfdb = IRCAM_LISTEN(r'D:\HRTF\IRCAM')
subject = 1002
hrtfset = hrtfdb.load_subject(subject)
index = randint(hrtfset.num_indices)
cfmin, cfmax, cfN = 150*Hz, 5*kHz, 40
sound = Sound.whitenoise(500*ms)

hrtf = hrtfset.hrtf[index]
cf = erbspace(cfmin, cfmax, cfN)
num_indices = hrtfset.num_indices

try:
    all_itds, all_ilds = pickle.load(open('itd_ild_data.pkl', 'r'))
    print 'Loaded pregenerated ITD/ILD data'
except IOError:
    print 'Performing ITD/ILD analysis (may take several minutes)'
    start = time.time()
    all_itds = []
    all_ilds = []
    for j in xrange(num_indices):
        hrirset = Sound(hrtfset.hrtf[j].fir.T, samplerate=hrtfset.samplerate)
        fb = GammatoneFilterbank(Repeat(hrirset, cfN),
                                 hstack((cf, cf)))
        fb.buffer_init()
        filtered_hrirset = fb.buffer_fetch(0, hrtfset.num_samples)
        itds = []
        ilds = []
        for i in xrange(cfN):
            left = filtered_hrirset[:, i]
            right = filtered_hrirset[:, i+cfN]
            Lf = fft(hstack((left, zeros(len(left)))))
            Rf = fft(hstack((right[::-1], zeros(len(right)))))
            C = ifft(Lf*Rf).real
            i = argmax(C)+1-len(left)
            itds.append(i/hrtfset.samplerate)
            ilds.append(sqrt(amax(C)/sum(right**2)))
        itds = array(itds)
        ilds = array(ilds)
        all_itds.append(itds)
        all_ilds.append(ilds)
    print 'Time taken:', time.time()-start
    pickle.dump((all_itds, all_ilds), open('itd_ild_data.pkl', 'w'))
    print 'File saved as itd_ild_data.pkl'

d = array([all_itds[j][i] for i in xrange(cfN) for j in xrange(num_indices)])
g = array([all_ilds[j][i] for i in xrange(cfN) for j in xrange(num_indices)])
gains = hstack((1/g, g))
gains_dB = 20*log10(gains)
abs_gains_dB = abs(gains_dB)
r = -abs_gains_dB[:len(gains)/2]
r = hstack((r, r))
gains_dB += r
gains = 10**(gains_dB/20)
gains = reshape(gains, (1, len(gains)))
delays_L = where(d>=0, zeros(len(d)), -d)
delays_R = where(d>=0, d, zeros(len(d)))
delay_max = max(amax(delays_L), amax(delays_R))*second

# The following actually defines the filters and neurons

hrtf_fb = hrtf.filterbank(sound)
gfb = GammatoneFilterbank(
        RestructureFilterbank(hrtf_fb, cfN),
        hstack((cf, cf)))
gains_fb = FunctionFilterbank(
        RestructureFilterbank(gfb, num_indices),
        lambda x:x*gains,
        )
cochlea = FunctionFilterbank(gains_fb, lambda x:15*clip(x, 0, Inf)**(1.0/3.0))

eqs = '''
dV/dt = (I-V)/(1*ms)+0.1*xi/(0.5*ms)**.5 : 1
I : 1
'''
G = FilterbankGroup(cochlea, 'I', eqs, reset=0, threshold=1, refractory=5*ms)
cd = NeuronGroup(num_indices*cfN, eqs, reset=0, threshold=1, clock=G.clock)

C = Connection(G, cd, 'V', delay=True, max_delay=delay_max)
for i in xrange(num_indices*cfN):
    C[i, i] = 0.5
    C[i+num_indices*cfN, i] = 0.5
    C.delay[i, i] = delays_L[i]
    C.delay[i+cfN*num_indices, i] = delays_R[i]

M = SpikeMonitor(G, record=sound.duration<50*ms)
Mcd = SpikeMonitor(cd, record=sound.duration<50*ms)
counter = SpikeCounter(cd)

start = time.time()
run(sound.duration, report='stderr')
print 'Time taken:', time.time()-start

count = counter.count

print 'Input spike rate:', float(M.nspikes)/(len(G)*G.clock.t)
print 'Coincidence spike rate:', float(Mcd.nspikes)/(len(cd)*G.clock.t)

if sound.duration<50*ms:
    subplot(221)
    raster_plot(M)
    subplot(223)
    raster_plot(Mcd)
    subplot(122)
ircam_plot_count(hrtfset, count, index=index)
show()



from brian import *
from filterbank import *
from firfilterbank import *

__all__ = ['FractionalDelay']

class FractionalDelay(FIRFilterbank):
    '''
    Filterbank for applying delays which are fractional multiples of the timestep
    
    Initialised with arguments:
    
    ``source``
        Source sound or filterbank.
    ``delays``
        A list or array of delays to apply (the number of channels in the
        filterbank will be equal to the length of this).
    ``filter_length=None``
        Use this to explicitly set the length of the impulse response, should
        be odd. If not specified, it will be automatically determined from
        the delays. See notes below.
    ``**args``
        Arguments to pass to :class:`FIRFilterbank` (from which this class
        is derived).
        
    **Attributes**
    
    .. attribute:: delay_offset
    
        The global delay offset. If the specified delay in a given channel is
        ``delay`` the actual delay will be ``delay_offset+delay``. It is equal
        to ``(filter_length/2)/source.samplerate``.
        
    .. attribute:: filter_length
    
        The length of the filter to use. This is automatically determined
        from the delays. Note that ``delay_offset`` should be larger than the
        maximum positive or negative delay. The minimum filter length is
        by default 2048 samples, which allows for good accuracy for signals
        with power above 20 Hz. For low frequency analysis, longer filters will
        be necessary. For high frequency analysis, a shorter filter length could
        be used for a more efficient computation.
    
    **Notes**
    
    Inducing a delay for a sound that is an integer multiple of the timestep
    (1/samplerate) can be done simply by offsetting the samples, e.g.
    ``sound[3:]`` is ``sound`` delayed by ``3/sound.samplerate``. However,
    for fractional multiples of the timestep, the sound needs to be filtered.
    The theory and code for this was adapted from
    `http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html <http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html>`__.
    
    The filters induce a delay of ``delay_offset+delay`` where ``delay_offset``
    is a positive value larger than the maximum positive or negative delay.
    This value is available as the attribute ``delay_offset``.
    '''
    def __init__(self, source, delays, filter_length=None, **args):
        delays = array(delays)
        delay_max = amax(abs(delays))
        delay_max_int = int(ceil(source.samplerate*delay_max))
        if filter_length is None:
            filter_length = 2*int(delay_max_int*1.25)+1
            if filter_length<2048:
                filter_length = 2048
        if filter_length/2<=delay_max_int:
            raise ValueError('Filter length not long enough for selected delays.')
        self.delay_offset = (filter_length//2)/source.samplerate
        self.filter_length = filter_length
        self.delays = delays
        irs = [fractional_delay_ir(delay, source.samplerate,
                    filter_length=filter_length) for delay in delays]
        irs = array(irs)
        self.impulse_response = irs
        FIRFilterbank.__init__(self, source, irs, **args)

# Adapted from
# http://www.labbookpages.co.uk/audio/beamforming/fractionalDelay.html    
def fractional_delay_ir(delay, samplerate, filter_length=151):
    delay = delay*samplerate
    centre_tap = filter_length // 2
    t = arange(filter_length)
    x = t-delay
    if abs(round(delay)-float(delay))<1e-10:
        return array(x==centre_tap, dtype=float)
    sinc = sin(pi*(x-centre_tap))/(pi*(x-centre_tap))
    window = 0.54-0.46*cos(2.0*pi*(x+0.5)/filter_length) # Hamming window
    tap_weight = window*sinc
    return tap_weight



'''
Automatic differentiation.
R. Brette
---------
Method: forward accumulation by operator overloading.
TODO:
* make it work with vectors (i.e., jacobian matrix)
* iadd, etc.
* NonDifferentiableException and maybe use this on conditions (when cmp=0)
* Differential operators: Laplacian...
* Interfacing with Scipy optimization algorithms.
* Check lecture notes on AD: http://www-sop.inria.fr/tropics/tropics/support_cours_d_a.pdf

The differentiate function can be used to calculate the
(possibly higher-order) partial derivatives of a function
with respect to any variable (e.g. d2 f/dx dy).

It is working with scalar variables for the moment, but
there is some limited functionality for vectors, i.e., the
differentiation should work for functions with vectorial arguments
if only scalar operations are used, e.g.:
def f(x):
    y=0
    for xi in x:
        y+=xi*xi
    return sqrt(y)
Then differentiate(f,array([1,2,3,4])) returns the gradient of f
at point x=array([1,2,3,4]), as a sparse matrix (lil_matrix type).

If several partial derivatives of the same function need to be
calculated, it is better to use the lower level object Differentiable.
Example:
y=f(Differentiable('x',3),Differentiable('y',2))
will return a Differentiable object with
y.val=f(3,2)
and y.diff={'x': df/dx(3,2), 'y': df/dy(3,2)}

Also implemented:
* Taylor series
* Gradient
* Hessian
* Works with units
'''

from numpy import ndarray, exp, sin, cos, log, zeros
from scipy import sparse
import types

__all__ = ['differentiate', 'Differentiable', 'taylor_series', 'gradient', 'hessian']

def differentiate(f, x=None, order=1):
    '''
    Calculate the derivative of f at point x.
    Higher-order derivatives are possible.
    Ex.: differentiate(lambda x:3*x*x+2,x=2,order=2)
    (Returns: 6.0)
    
    Partial derivatives:
    1) Give a tuple with the order.
    Ex.: differentiate(lambda x,y:x*y+2*y+1,x=(1,2),order=(0,1))
    (Returns: 3.0 (partial derivative d/dy) )
    2) Give a dictionnary with the order:
    Ex.: differentiate(lambda x,y:x*y+2*y+1,x=(1,2),order={'y':1})
    '''
    if x is None:
        return lambda x:differentiate(f, x, order)
    if (type(order) == types.ListType) or (type(order) == types.TupleType): # several variables
        # Build the list of arguments
        n = sum(order) # total order
        args = [HigherDifferentiable(i, x[i], n) for i in range(len(x))]
        y = f(*args)
        for i in range(len(x)):
            for j in range(order[i]):
                if isinstance(y, Differentiable) and (i in y.diff):
                    y = y.diff[i]
                else: # constant
                    return 0.
        return y
    elif (type(order) == types.DictType): # named variables
        # Build the list of arguments
        n = sum(order.itervalues())
        vars = list(f.func_code.co_varnames)
        args = []
        args.extend(x)
        for name in order.iterkeys():
            i = vars.index(name)
            args[i] = HigherDifferentiable(name, x[i], n)
        y = f(*args)
        for name, n in order.iteritems():
            for j in range(n):
                if isinstance(y, Differentiable) and (name in y.diff):
                    y = y.diff[name]
                else: # constant
                    return 0.
        return y
    elif order == 0:
        return f(x)
    elif order == 1:
        y = f(Differentiable('x', x))
        if isinstance(y, Differentiable) and ('x' in y.diff):
            return y.diff['x']
        else: # constant
            return 0.
    else:
        return differentiate(lambda y:differentiate(f, y, order=order - 1), x)

def taylor_series(f, x, order=1):
    '''
    Returns the list of coefficients of the Taylor Series of f
    at x (scalar).
    '''
    y = f(HigherDifferentiable('x', x, order))
    series = []
    n = 1.
    for i in range(order + 1):
        # Value
        val = y
        while isinstance(val, Differentiable):
            val = val.val
        series.append(val / n)
        n *= (i + 1)
        # Next order
        if isinstance(y, Differentiable) and ('x' in y.diff):
            y = y.diff['x']
        else:
            y = 0.
    return series

def gradient(f, x):
    '''
    Gradient of f at x (= tuple).
    Result = list.
    '''
    args = [Differentiable(i, val=x[i]) for i in range(len(x))]
    y = f(*args)
    if isinstance(y, Differentiable):
        result = []
        for i in range(len(x)):
            if i in y.diff:
                result.append(y.diff[i])
            else:
                result.append(0.)
        return result
    else: # constant
        return [0.] * len(x)

def hessian(f, x):
    '''
    Hessian of f at x (= tuple).
    Result = array.
    N.B.: not working with vectors.
    '''
    args = [HigherDifferentiable(i, x[i], 2) for i in range(len(x))]
    y = f(*args)
    result = zeros((len(x), len(x)))
    if isinstance(y, Differentiable): # non-zero Hessian
        for i in range(len(x)):
            if i in y.diff and isinstance(y.diff[i], Differentiable):
                for j in range(len(x)):
                    if j in y.diff[i].diff:
                        result[i, j] = y.diff[i].diff[j]
    return result


class Differentiable(object):
    '''
    A differentiable variable.
    Implemented operations: +,-,*,[],len,exp,sin,cos,/,abs,sqrt,comparisons,**
    '''
    def __init__(self, name=None, val=0.):
        '''
        Initializes a variable and its derivative.
        If the name is None, then it is a constant.
        '''
        self.val = val # value
        if name == None: # constant
            self.diff = {} # derivative
        #elif isinstance(val,ndarray): # a vector
            # TODO: different class? (DifferentiableVector)
            #self.diff={name:eye_lil_matrix(len(val))} # Jacobian matrix
        else:
            self.diff = {name:1.} # or jacobian? (eye)

    def sqrt(self):
        return self ** .5

    def __cmp__(self, x):
        if isinstance(x, Differentiable):
            return cmp(self.val, x.val)
        else:
            return cmp(self.val, x)

    def __abs__(self):
        # NOT WORKING WITH VECTORS
        if self.val == 0:
            # TODO: specific exception
            raise ZeroDivisionError, "x:abs(x) is not differentiable at x=0."
        elif self.val > 0:
            return self
        else:
            return - self

    def __add__(self, y):
        # VECTOR-READY
        if isinstance(y, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(y.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in y.diff):
                    zdict[key] = self.diff[key] + y.diff[key]
            z = Differentiable()
            z.val = self.val + y.val
            z.diff = zdict
            return z
        else: # Adding a constant
            z = Differentiable(val=self.val + y)
            z.diff = self.diff
            return z

    def __radd__(self, x):
        # VECTOR-READY
        if isinstance(x, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(x.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in x.diff):
                    zdict[key] = x.diff[key] + self.diff[key]
            z = Differentiable()
            z.val = x.val + self.val
            z.diff = zdict
            return z
        else: # Adding a constant
            z = Differentiable(val=x + self.val)
            z.diff = self.diff
            return z

    def __mul__(self, y):
        # !Not working with vectors!
        if isinstance(y, Differentiable):
            #TODO: with vectors
            zdict = {}
            zdict.update(self.diff)
            zdict.update(y.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in y.diff):
                    zdict[key] = self.diff[key] * y.val + self.val * y.diff[key]
                elif (key in self.diff):
                    zdict[key] = self.diff[key] * y.val
                else:
                    zdict[key] = self.val * y.diff[key]
            z = Differentiable()
            z.val = self.val * y.val
            z.diff = zdict
            return z
        else: # Multiplying by a constant
            z = Differentiable(val=self.val * y)
            for key in self.diff:
                z.diff[key] = self.diff[key] * y
            return z

    def __rmul__(self, x):
        if isinstance(x, Differentiable):
            #TODO: with vectors
            zdict = {}
            zdict.update(self.diff)
            zdict.update(x.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in x.diff):
                    zdict[key] = x.val * self.diff[key] + x.diff[key] * self.val
                elif (key in self.diff):
                    zdict[key] = x.val * self.diff[key]
                else:
                    zdict[key] = x.diff[key] * self.val
            z = Differentiable()
            z.val = x.val * self.val
            z.diff = zdict
            return z
        else: # Multiplying by a constant
            z = Differentiable(val=x * self.val)
            for key in self.diff:
                z.diff[key] = x * self.diff[key]
            return z

    def __div__(self, x):
        return self * (x ** -1)

    def __rdiv__(self, x):
        return x * (self ** -1)

    def __sub__(self, y):
        # VECTOR-READY
        if isinstance(y, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(y.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in y.diff):
                    zdict[key] = self.diff[key] - y.diff[key]
            z = Differentiable()
            z.val = self.val - y.val
            z.diff = zdict
            return z
        else: # Subtracting a constant
            z = Differentiable(val=self.val - y)
            z.diff = self.diff
            return z

    def __rsub__(self, x):
        # VECTOR-READY
        if isinstance(x, Differentiable):
            zdict = {}
            zdict.update(self.diff)
            zdict.update(x.diff)
            for key in zdict.iterkeys():
                if (key in self.diff) and (key in x.diff):
                    zdict[key] = x.diff[key] - self.diff[key]
            z = Differentiable()
            z.val = x.val - self.val
            z.diff = zdict
            return z
        else: # Subtracting a constant
            z = Differentiable(val=x - self.val)
            for key in self.diff:
                z.diff[key] = -self.diff[key]
            return z

    def __neg__(self):
        # VECTOR-READY
        z = Differentiable(val= -self.val)
        for key in self.diff:
            z.diff[key] = -self.diff[key]
        return z

    def __pow__(self, x):
        # NOT WORKING WITH VECTORS
        if isinstance(x, Differentiable):
            return exp(x * log(self))
        elif x == 0:
            z = Differentiable(val=self.val ** x)
            for key in self.diff:
                z.diff[key] = 0 * self.diff[key]
            return z
        else:
            z = Differentiable(val=self.val ** x)
            for key in self.diff:
                z.diff[key] = x * self.val ** (x - 1) * self.diff[key]
            return z

    def __rpow__(self, x):
        # NOT WORKING WITH VECTORS
        return exp(self * log(x))

    def __getitem__(self, i):
        z = Differentiable(val=self.val[i])
        z.diff = {}
        for key in self.diff.iterkeys():
            z.diff[key] = self.diff[key][i, :]
        return z

    def __len__(self):
        return len(self.val)

    def __str__(self):
        s = str(self.val)
        for key, value in self.diff.iteritems():
            s += ' ; d/d' + key + '=' + str(value)
        return s

    def exp(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = exp(self.val) * self.diff[key]
        z = Differentiable(val=exp(self.val))
        z.diff = zdict
        return z

    def cos(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = -sin(self.val) * self.diff[key]
        z = Differentiable(val=cos(self.val))
        z.diff = zdict
        return z

    def sin(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = cos(self.val) * self.diff[key]
        z = Differentiable(val=sin(self.val))
        z.diff = zdict
        return z

    def log(self):
        # !! Not working for vectors !!
        zdict = {}
        zdict.update(self.diff)
        for key in zdict.iterkeys():
            zdict[key] = (1. / (self.val)) * self.diff[key]
        z = Differentiable(val=log(self.val))
        z.diff = zdict
        return z


class HigherDifferentiable(Differentiable):
    '''
    A differentiable variable with order n.
    '''
    def __init__(self, name=None, val=0., order=1):
        Differentiable.__init__(self, name, val)
        if order > 1:
            self.val = HigherDifferentiable(name, val, order - 1)

#Utility function
def eye_lil_matrix(n):
    '''
    Returns the identity matrix of size n as a lil_matrix
    (sparse matrix).
    '''
    M = sparse.lil_matrix((n, n))
    M.setdiag([1.] * n)
    return M



from brian import *

G = NeuronGroup(4, 'V:1')
Ga = G.subgroup(2)
Gb = G.subgroup(2)

C = Connection(G, G, 'V', delay=True, structure='dense')

C.connect_full(Ga, Gb, weight=1, delay=lambda:rand())

print C.W.todense()
print C.delayvec.todense()



from numpy import *
lines = open('brian_docs_analysis.txt', 'r').read().split('\n')
lines = [line for line in lines if '\t' in line]
header = lines[0].split('\t')
lines = [tuple(line.split('\t')) for line in lines[1:]]
logdtype = [('page', object),
            ('visits', int),
            ('views', int),
            ('duration', float),
            ('sorties', float),
            ('value', float)]
print header
data = array(lines, dtype=logdtype)
data.sort(order='visits')
data = data[::-1]

def write_data(data, fname, n=100):
    with open(fname, 'w') as f:
        f.write('\t'.join(map(str, header))+'\n')
        for d in data[:n]:
            f.write('\t'.join(map(str, d))+'\n')

docs_data = data[array([d.startswith('/docs/') for d in data['page']])]
write_data(docs_data, 'topdocs_visits.txt')

docs_data_html = docs_data[array([d.endswith('.html') for d in docs_data['page']])]
write_data(docs_data_html, 'topdocs_visits_html.txt')

I = argsort(-docs_data_html['views']*1.0/docs_data_html['visits'])
docs_data_html_revisited = docs_data_html[I]
docs_data_html_revisited = docs_data_html_revisited[array([d>20 for d in docs_data_html_revisited['views']])]
write_data(docs_data_html_revisited, 'topdocs_revisited_html.txt')

I = argsort(-docs_data_html['duration'])
docs_data_html_duration = docs_data_html[I]
docs_data_html_duration = docs_data_html_duration[array([d>20 for d in docs_data_html_duration['views']])]
write_data(docs_data_html_duration, 'topdocs_duration_html.txt')



from brian import *

eqs1 = '''
dV/dt = -V/(10*ms) : 1
w = 10*V : 1
'''

eqs2 = '''
V : 1
w = 10*V : 1
'''

G1 = NeuronGroup(1, eqs1)
G2 = NeuronGroup(1, eqs2)

G1.V = 1
G2.V = linked_var(G1, 'w')

print G1.staticvars
print G2.staticvars

M = StateMonitor(G2, 'V', record=True)

run(30*ms)

M.plot()
show()



from brian import *

tau=10*ms
x1=-1*volt
x2=0*volt
model='dv/dt=((v>0)*x1+(v<=0)*x2)/tau:volt'
G=NeuronGroup(1,model,method='Euler')
S=StateMonitor(G,'v',record=0)
G.v=1*volt
run(50*ms)

S.plot()
show()



'''
Runs all of the examples in the examples/ folder.

This version 

Note: removes all show() commands so that you don't have to wait for user input.
'''

import os, glob, sys, StringIO, time, gc, fnmatch
import brian

exclude_list = open('examples_exclude.txt', 'r').read().split('\n')
exclude_list = [f for f in exclude_list if not f.startswith('#') and f]

if os.path.exists('examples_completed.txt'):
    examples_completed = [f for f in open('examples_completed.txt', 'r').read().split('\n') if f]
else:
    examples_completed = []

os.chdir('../../examples')


class GlobDirectoryWalker:
    # a forward iterator that traverses a directory tree

    def __init__(self, directory, pattern="*"):
        self.stack = [directory]
        self.pattern = pattern
        self.files = []
        self.index = 0

    def __getitem__(self, index):
        while 1:
            try:
                file = self.files[self.index]
                self.index = self.index + 1
            except IndexError:
                # pop next directory from stack
                self.directory = self.stack.pop()
                self.files = os.listdir(self.directory)
                self.index = 0
            else:
                # got a filename
                fullname = os.path.join(self.directory, file)
                if os.path.isdir(fullname) and not os.path.islink(fullname):
                    self.stack.append(fullname)
                if fnmatch.fnmatch(file, self.pattern):
                    return fullname

examplefilenames = [name for name in GlobDirectoryWalker('', '*.py') if all(fname not in name for fname in exclude_list)]
exceptions = []
starttime = time.time()
stdout = sys.stdout
stderr, sys.stderr = sys.stderr, StringIO.StringIO()
for fname in examplefilenames:
    if fname not in examples_completed:
        try:
            code = open(fname, 'r').read().replace('show()', '')
            ns = {}
            print 'Running example', fname,
            brian.reinit_default_clock()
            sys.stdout = StringIO.StringIO()
            exec code in ns
            del ns
            gc.collect()
            sys.stdout = stdout
            print '[ok]'
            examples_completed.append(fname)
            open('../dev/tests/examples_completed.txt', 'w').write('\n'.join(examples_completed))
        except Exception, inst:
            sys.stdout = stdout
            print '[error]'
            exceptions.append((fname, inst))
sys.stderr = stderr
endtime = time.time()
print
if exceptions:
    print 'Exceptions encountered:'
    for fname, inst in exceptions:
        print 'In file', fname, '-', inst
    print
    print len(examplefilenames) - len(exceptions), 'of', len(examplefilenames), 'ran OK, time taken', endtime - starttime, 'seconds'
else:
    print 'OK, time taken', endtime - starttime, 'seconds'



'''
Network (CUBA) with short-term synaptic plasticity for excitatory synapses
(Depressing at long timescales, facilitating at short timescales)
'''
from brian import *
from time import time

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(4000, model=eqs, threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + rand(4000) * 10 * mV
Pe = P.subgroup(3200)
Pi = P.subgroup(800)
Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=.02)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=.02)
stp = STP(Ce, taud=200 * ms, tauf=20 * ms, U=.2)
M = SpikeMonitor(P)
rate = PopulationRateMonitor(P)
t1 = time()
run(1 * second)
t2 = time()
print "Simulation time:", t2 - t1, "s"
print M.nspikes, "spikes"
subplot(211)
raster_plot(M)
subplot(212)
plot(rate.times / ms, rate.smooth_rate(5 * ms))
show()



#!/usr/bin/env python

"""
setup.py file for SWIG example
"""

from distutils.core import setup, Extension
import os, numpy
numpy_base_dir = os.path.split(numpy.__file__)[0]
numpy_include_dir = os.path.join(numpy_base_dir, 'core/include')

brianlib_module = Extension('_brianlib',
                           sources=['brianlib_wrap.cxx',
                                    'circular.cpp',
                                    'monitor.cpp',
                                    'neurongroup.cpp',
                                    'network.cpp',
                                    'reset.cpp',
                                    'stateupdater.cpp',
                                    'threshold.cpp',
                                    'connection.cpp',
                                    ],
                           include_dirs=[numpy_include_dir],
                           extra_compile_args=['-O3']
                           )

setup (name='brianlib',
       version='0.1',
       author="Dan Goodman",
       description="""Simple swig example from docs""",
       ext_modules=[brianlib_module],
       py_modules=["brianlib"],
       )



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

hrtfdb = IRCAM_LISTEN(r'D:\HRTF\IRCAM')
subject = 1002
hrtfset = hrtfdb.load_subject(subject)
hrtf = hrtfset.hrtf[0]

x = Sound(randn(10000, 1), samplerate=44100*Hz)
y = hrtf.apply(x)

#fb = FIRFilterbank(x, hrtf.impulse_response.T)
fb = hrtf.filterbank(x)
fb.buffer_init()
print 'First fetch'
import time
start = time.time()
z1 = fb.buffer_fetch(0, 5000)
print 'Second fetch'
print 'time:', time.time()-start
z2 = fb.buffer_fetch(5000, 7500)
print 'Third fetch'
z3 = fb.buffer_fetch(7500, 10000)
print 'Finished fetching'
z = vstack((z1, z2, z3))

print z.shape

print amax(abs(z-y))

subplot(211)
plot(x)
subplot(212)
plot(y)
show()



'''
Very short example program.

Profiling:
* state update = 0.17 s, including 0.11 s in state updater
  but only 0.016 if no network update
* threshold = 0.22 s
* reset = 0.06 s
* each connection = about 0.20 s (even with no spike)
'''
#import brian_no_units
from brian import *
from time import time

N = 5        # number of neurons
Ne = int(N * 0.8) # excitatory neurons 
Ni = N - Ne       # inhibitory neurons
p = .1
duration = 1000 * ms

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''

P = NeuronGroup(N, model=eqs)#,threshold=-50*mV,reset=-60*mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
#P._state_updater=lambda _:0
#Pe=P.subgroup(Ne)
#Pi=P.subgroup(Ni)

#Ce=Connection(Pe,P,'ge',weight=0*mV,sparseness=p)
#Ci=Connection(Pi,P,'gi',weight=0*mV,sparseness=p)

run(1 * ms)
t1 = time()
run(duration)
t2 = time()
print "Simulated in", t2 - t1, "s"



"""
Coincidence detection example
=============================
Fig. 4 from:

    Rossant C, Leijon S, Magnusson AK, Brette R (2011).
    "Sensitivity of noisy neurons to coincident inputs".
    Journal of Neuroscience, 31(47).

Two distant or coincident spikes are injected into a noisy balanced
leaky integrate-and-fire neuron. The PSTH of the neuron in response to 
these inputs is calculated along with the extra number of spikes 
in the two cases. This number is higher for the coincident spikes,
showing the sensitivity of a noisy neuron to coincident inputs.
"""
from brian import *
import matplotlib.patches as patches
import matplotlib.path as path

def histo(bins, cc, ax):
    # get the corners of the rectangles for the histogram
    left = array(bins[:-1])
    right = array(bins[1:])
    bottom = zeros(len(left))
    top = bottom + cc
    
    # we need a (numrects x numsides x 2) numpy array for the path helper
    # function to build a compound path
    XY = array([[left,left,right,right], [bottom,top,top,bottom]]).T
    
    # get the Path object
    barpath = path.Path.make_compound_path_from_polys(XY)
    
    # make a patch out of it
    patch = patches.PathPatch(barpath, facecolor='blue', edgecolor='gray', alpha=0.8)
    ax.add_patch(patch)
    
    # update the view limits
    ax.set_xlim(left[0], right[-1])
    ax.set_ylim(bottom.min(), top.max())

# neuron parameters
theta = -55*mV
vmean = -65*mV
taum = 5*ms
taue = 3*ms
taun = 15*ms
sigma = 4*mV

# input times
t1 = 100*ms
t2 = 120*ms

# simulation duration
dur = 200*ms

# number of neuron
N = 10000
bin = 2*ms

# EPSP size
int_EPSP=taue
int_EPSP2=taue*taue/(2*(taum+taue))
max_EPSP=(taum/taue)**(taum/(taue-taum))
we = 3.0*mV/max_EPSP

# model equations
eqs = '''
V=V0+noise : volt
dV0/dt=(-V0+psp)/taum : volt
dpsp/dt=-psp/taue : volt
dnoise/dt=(vmean-noise)/taun+sigma*(2./taun)**.5*xi : volt
'''
threshold = 'V>theta'
reset = vmean

# initialization of the NeuronGroup
reinit_default_clock()
group = NeuronGroup(2*N, model=eqs, reset=reset, threshold=threshold)
group.V0 = group.psp = 0*volt
group.noise = vmean + sigma * randn(2*N)

# input spikes
input_spikes = [(0, t1), (0, t2), (1, t1)]
input = SpikeGeneratorGroup(2, array(input_spikes))

# connections
C = Connection(input, group, 'psp')
C.connect_full(input[0], group[:N], weight=we)
C.connect_full(input[1], group[N:], weight=2*we)

# monitors
prM1 = PopulationRateMonitor(group[:N], bin=bin)
prM2 = PopulationRateMonitor(group[N:], bin=bin)

# launch simulation
run(dur)

# PSTH plot
figure(figsize=(10,10))
prMs = [prM1, prM2]
for i in [0,1]:
    prM = prMs[i]
    r = prM.rate[:-1]*bin
    m = mean(r[:len(r)/2])

    ax = subplot(211+i)
    histo(prM.times, r, ax)
    plot([0,dur],[m,m],'--r')
    title("%.2f extra spikes" % sum(r[t1/bin:(t2+20*ms)/bin]-m))
    xlim(.05, .2)
    ylim(0, .125)

show()



"""
Generates the file dist/brian-versionnumber-extras.zip from the folders defined in the
setup.py module
"""

import os
import glob
from setup import version, extras_folders
from zipfile import ZipFile

zipfilename = 'dist/brian-' + version + '-extras.zip'

os.chdir('..')
os.chdir('..')
os.chdir('..') # work from Brian's root

files = []
for folder in extras_folders:
    files.extend(glob.glob(folder))
zfile = ZipFile(zipfilename, 'w')
for file in files:
    zfile.write(file)
zfile.close()



from base import *
from sparsematrix import *
from connectionvector import *
from constructionmatrix import *
from connectionmatrix import *
from construction import *
from connection import *
from propagation_c_code import *
import warnings
network_operation = None # we import this when we need to because of order of import issues

__all__ = [
         'DelayConnection',
         ]


class DelayConnection(Connection):
    '''
    Connection which implements heterogeneous postsynaptic delays
    
    Initialised as for a :class:`Connection`, but with the additional
    keyword:
    
    ``max_delay``
        Specifies the maximum delay time for any
        neuron. Note, the smaller you make this the less memory will be
        used.
    
    Overrides the following attribute of :class:`Connection`:
    
    .. attribute:: delay
    
        A matrix of delays. This array can be changed during a run,
        but at no point should it be greater than ``max_delay``.
    
    In addition, the methods ``connect``, ``connect_random``, ``connect_full``,
    and ``connect_one_to_one`` have a new keyword ``delay=...`` for setting the
    initial values of the delays, where ``delay`` can be one of:

    * A float, all delays will be set to this value
    * A pair (min, max), delays will be uniform between these two
      values.
    * A function of no arguments, will be called for each nonzero
      entry in the weight matrix.
    * A function of two argument ``(i,j)`` will be called for each
      nonzero entry in the weight matrix.
    * A matrix of an appropriate type (e.g. ndarray or lil_matrix).

    Finally, there is a method:
    
    ``set_delays(source, target, delay)``
        Where ``delay`` must be of one of the types above.
    
    **Notes**
    
    This class implements post-synaptic delays. This means that the spike is
    propagated immediately from the presynaptic neuron with the synaptic
    weight at the time of the spike, but arrives at the postsynaptic neuron
    with the given delay. At the moment, Brian only provides support for
    presynaptic delays if they are homogeneous, using the ``delay`` keyword
    of a standard ``Connection``.
    
    **Implementation**
    
    :class:`DelayConnection` stores an array of size ``(n,m)`` where
    ``n`` is ``max_delay/dt`` for ``dt`` of the target :class:`NeuronGroup`'s clock,
    and ``m`` is the number of neurons in the target. This array can potentially
    be quite large. Each row in this array represents the array that should be
    added to the target state variable at some particular future time. Which
    row corresponds to which time is tracked using a circular indexing scheme.
    
    When a spike from neuron ``i`` in the source is encountered, the delay time
    of neuron ``i`` is looked up, the row corresponding to the current time
    plus that delay time is found using the circular indexing scheme, and then
    the spike is propagated to that row as for a standard connection (although
    this won't be propagated to the target until a later time).
    
    **Warning**
    
    If you are using a dynamic connection matrix, it is your responsibility to
    ensure that the nonzero entries of the weight matrix and the delay matrix
    exactly coincide. This is not an issue for sparse or dense matrices.
    '''

    def __init__(self, source, target, state=0, modulation=None,
                 structure='sparse',
                 weight=None, sparseness=None, delay=None,
                 max_delay=5 * msecond, **kwds):
        global network_operation
        if network_operation is None:
            from ..network import network_operation
        Connection.__init__(self, source, target, state=state, modulation=modulation,
                            structure=structure, weight=weight, sparseness=sparseness, **kwds)
        self._max_delay = int(max_delay / target.clock.dt) + 1
        source.set_max_delay(max_delay)
        # Each row of the following array stores the cumulative effect of spikes at some
        # particular time, defined by a circular indexing scheme. The _cur_delay_ind attribute
        # stores the row corresponding to the current time, so that _cur_delay_ind+1 corresponds
        # to that time + target.clock.dt, and so on. When _cur_delay_ind reaches _max_delay it
        # resets to zero.
        self._delayedreaction = numpy.zeros((self._max_delay, len(target)))
        # vector of delay times, can be changed during a run
        if isinstance(structure, str):
            structure = construction_matrix_register[structure]
        self.delayvec = structure((len(source), len(target)), **kwds)
        self._cur_delay_ind = 0
        # this network operation is added to the Network object via the contained_objects
        # protocol (see the line after the function definition). The standard Connection.propagate
        # function propagates spikes to _delayedreaction rather than the target, and this
        # function which is called after the usual propagations propagates that data from
        # _delayedreaction to the target. It only needs to be called each target.clock update.
        @network_operation(clock=target.clock, when='after_connections')
        def delayed_propagate():
            # propagate from _delayedreaction -> target group
            target._S[self.nstate] += self._delayedreaction[self._cur_delay_ind, :]
            # reset the current row of _delayedreaction
            self._delayedreaction[self._cur_delay_ind, :] = 0.0
            # increase the index for the circular indexing scheme
            self._cur_delay_ind = (self._cur_delay_ind + 1) % self._max_delay
        self.contained_objects = [delayed_propagate]
        # this is just used to convert delayvec's which are in ms to integers, precalculating it makes it faster
        self._invtargetdt = 1 / self.target.clock._dt
        self._useaccel = get_global_preference('useweave')
        self._cpp_compiler = get_global_preference('weavecompiler')
        self._extra_compile_args = ['-O3']
        if self._cpp_compiler == 'gcc':
            self._extra_compile_args += get_global_preference('gcc_options') # ['-march=native', '-ffast-math']
        if delay is not None:
            self.set_delays(delay=delay)

    def propagate(self, spikes):
        if not self.iscompressed:
            self.compress()
        if len(spikes):
            # Target state variable
            dr = self._delayedreaction
            # If specified, modulation state variable
            if self._nstate_mod is not None:
                sv_pre = self.source._S[self._nstate_mod]
            # Get the rows of the connection matrix, each row will be either a
            # DenseConnectionVector or a SparseConnectionVector.
            rows = self.W.get_rows(spikes)
            dvecrows = self.delayvec.get_rows(spikes)
            if not self._useaccel: # Pure Python version is easier to understand, but slower than C++ version below
                if isinstance(rows[0], SparseConnectionVector):
                    if self._nstate_mod is None:
                        # Rows stored as sparse vectors without modulation
                        for row, dvecrow in izip(rows, dvecrows):
                            if not len(row.ind) == len(dvecrow.ind):
                                raise RuntimeError('Weight and delay matrices must be kept in synchrony for sparse matrices.')
                            drind = (self._cur_delay_ind + numpy.array(self._invtargetdt * dvecrow, dtype=int)) % self._max_delay
                            dr[drind, dvecrow.ind] += row
                    else:
                        # Rows stored as sparse vectors with modulation
                        for i, row, dvecrow in izip(spikes, rows, dvecrows):
                            if not len(row.ind) == len(dvecrow.ind):
                                raise RuntimeError('Weight and delay matrices must be kept in synchrony for sparse matrices.')
                            drind = (self._cur_delay_ind + numpy.array(self._invtargetdt * dvecrow, dtype=int)) % self._max_delay
                            # note we call the numpy __mul__ directly because row is
                            # a SparseConnectionVector with different mul semantics
                            dr[drind, dvecrow.ind] += numpy.ndarray.__mul__(row, sv_pre[i])
                else:
                    if self._nstate_mod is None:
                        # Rows stored as dense vectors without modulation
                        drjind = numpy.arange(len(self.target), dtype=int)
                        for row, dvecrow in izip(rows, dvecrows):
                            drind = (self._cur_delay_ind + numpy.array(self._invtargetdt * dvecrow, dtype=int)) % self._max_delay
                            dr[drind, drjind[:len(drind)]] += row
                    else:
                        # Rows stored as dense vectors with modulation
                        drjind = numpy.arange(len(self.target), dtype=int)
                        for i, row, dvecrow in izip(spikes, rows, dvecrows):
                            drind = (self._cur_delay_ind + numpy.array(self._invtargetdt * dvecrow, dtype=int)) % self._max_delay
                            dr[drind, drjind[:len(drind)]] += numpy.ndarray.__mul__(row, sv_pre[i])
            else: # C++ accelerated code, does the same as the code above but faster and less pretty
                nspikes = len(spikes)
                cdi = self._cur_delay_ind
                idt = self._invtargetdt
                md = self._max_delay
                if isinstance(rows[0], SparseConnectionVector):
                    rowinds = [r.ind for r in rows]
                    datas = rows
                    if self._nstate_mod is None:
                        code = delay_propagate_weave_code_sparse
                        codevars = delay_propagate_weave_code_sparse_vars
                    else:
                        code = delay_propagate_weave_code_sparse_modulation
                        codevars = delay_propagate_weave_code_sparse_modulation_vars
                else:
                    if not isinstance(spikes, numpy.ndarray):
                        spikes = array(spikes, dtype=int)
                    N = len(self.target)
                    if self._nstate_mod is None:
                        code = delay_propagate_weave_code_dense
                        codevars = delay_propagate_weave_code_dense_vars
                    else:
                        code = delay_propagate_weave_code_dense_modulation
                        codevars = delay_propagate_weave_code_dense_modulation_vars
                weave.inline(code, codevars,
                             compiler=self._cpp_compiler,
                             type_converters=weave.converters.blitz,
                             extra_compile_args=self._extra_compile_args)

    def do_propagate(self):
        self.propagate(self.source.get_spikes(0))

    def _set_delay_property(self, val):
        self.delayvec[:] = val

    delay = property(fget=lambda self:self.delayvec, fset=_set_delay_property)

    def compress(self):
        if not self.iscompressed:
            # We want delayvec to have nonzero entries at the same places as
            # W does, so we use W to initialise the compressed version of
            # delayvec, and then copy the values from the old delayvec to
            # the new compressed one, allowing delayvec and W to not have
            # to be perfectly intersected at the initialisation stage. If
            # the structure is dynamic, it will be the user's
            # responsibility to update them in sequence
            delayvec = self.delayvec
            # Special optimisation for the lil_matrix case if the indices
            # already match. Note that this special optimisation also allows
            # you to use multiple synapses via a sort of hack (editing the
            # rows and data attributes of the lil_matrix explicitly)
            using_lil_matrix = False
            if isinstance(delayvec, sparse.lil_matrix):
                using_lil_matrix = True
            self.delayvec = self.W.connection_matrix(copy=True)
            repeated_index_hack = False
            for i in xrange(self.W.shape[0]):
                if using_lil_matrix:
                    try:
                        row = SparseConnectionVector(self.W.shape[1],
                                                     array(delayvec.rows[i], dtype=int),
                                                     array(delayvec.data[i]))
                        if sum(diff(row.ind)==0)>0:
                            warnings.warn("You are using the repeated index hack, be careful!")
                            repeated_index_hack = True
                        self.delayvec.set_row(i, row)
                    except ValueError:
                        if repeated_index_hack:
                            warnings.warn("You are using the repeated index "
                                          "hack and not ensuring that weight "
                                          "and delay indices correspond, this "
                                          "is almost certainly an error!")
                        using_lil_matrix = False
                        # The delayvec[i,:] operation for sparse.lil_matrix format
                        # is VERY slow, but the CSR format is fine.
                        delayvec = delayvec.tocsr()
                if not using_lil_matrix:
                    self.delayvec.set_row(i, array(todense(delayvec[i, :]), copy=False).flatten())
                
            Connection.compress(self)

    def set_delays(self, source=None, target=None, delay=None):
        '''
        Set the delays corresponding to the weight matrix
        
        ``delay`` must be one of:
        
        * A float, all delays will be set to this value
        * A pair (min, max), delays will be uniform between these two
          values.
        * A function of no arguments, will be called for each nonzero
          entry in the weight matrix.
        * A function of two argument ``(i,j)`` will be called for each
          nonzero entry in the weight matrix.
        * A matrix of an appropriate type (e.g. ndarray or lil_matrix).
        '''
        if delay is None:
            return
        W = self.W
        P = source or self.source
        Q = target or self.target
        i0, j0 = self.origin(P, Q)
        i1 = i0 + len(P)
        j1 = j0 + len(Q)
        if isinstance(W, sparse.lil_matrix):
            def getrow(i):
                inds = array(W.rows[i], dtype=int)
                inds = inds[logical_and(inds >= j0, inds < j1)]
                return inds, len(inds)
        else:
            def getrow(i):
                inds = (W[i, j0:j1] != 0).nonzero()[0] + j0
                return inds, len(inds)
                #return slice(j0, j1), j1-j0
        if isinstance(delay, (float, int)):
            for i in xrange(i0, i1):
                inds, L = getrow(i)
                self.delayvec[i, inds] = delay
        elif isinstance(delay, (tuple, list)) and len(delay) == 2:
            delaymin, delaymax = delay
            for i in xrange(i0, i1):
                inds, L = getrow(i)
                rowdelay = rand(L) * (delaymax - delaymin) + delaymin
                self.delayvec[i, inds] = rowdelay
        elif callable(delay) and delay.func_code.co_argcount == 0:
            for i in xrange(i0, i1):
                inds, L = getrow(i)
                rowdelay = [delay() for _ in xrange(L)]
                self.delayvec[i, inds] = rowdelay
        elif callable(delay) and delay.func_code.co_argcount == 2:
            for i in xrange(i0, i1):
                inds, L = getrow(i)
                if isinstance(inds, slice):
                    inds = numpy.arange(inds.start, inds.stop)
                self.delayvec[i, inds] = delay(i - i0, inds - j0)
        else:
            #raise TypeError('delays must be float, pair or function of 0 or 2 arguments')
            self.delayvec[i0:i1, j0:j1] = delay # probably won't work, but then it will raise an error

    def connect(self, source=None, target=None, W=None, delay=None):
        Connection.connect(self, source=source, target=target, W=W)
        if delay is not None:
            self.set_delays(source, target, delay)

    def connect_random(self, source=None, target=None, p=1.0, weight=1.0,
                       fixed=False, seed=None, sparseness=None, delay=None):
        Connection.connect_random(self, source=source, target=target, p=p,
                                  weight=weight, fixed=fixed, seed=seed,
                                  sparseness=sparseness)
        if delay is not None:
            self.set_delays(source, target, delay)

    def connect_full(self, source=None, target=None, weight=1.0, delay=None):
        Connection.connect_full(self, source=source, target=target, weight=weight)
        if delay is not None:
            self.set_delays(source, target, delay)

    def connect_one_to_one(self, source=None, target=None, weight=1.0, delay=None):
        Connection.connect_one_to_one(self, source=source, target=target,
                                      weight=weight)
        if delay is not None:
            self.set_delays(source, target, delay)



'''
Example of the use of the class :class:`~brian.hears.LogGammachirp` available in
the library. It implements a filterbank of IIR gammachirp filters as 
Unoki et al. 2001, "Improvement of an IIR asymmetric compensation gammachirp
filter". In this example, a white noise is filtered by a linear gammachirp
filterbank and the resulting cochleogram is plotted. The different impulse
responses are also plotted.
'''
from brian import *
from brian.hears import *

sound = whitenoise(100*ms).ramp()
sound.level = 50*dB

nbr_center_frequencies = 50  #number of frequency channels in the filterbank

c1 = -2.96 #glide slope
b1 = 1.81  #factor determining the time constant of the filters

#center frequencies with a spacing following an ERB scale
cf = erbspace(100*Hz, 1000*Hz, nbr_center_frequencies)

gamma_chirp = LogGammachirp(sound, cf, c=c1, b=b1) 

gamma_chirp_mon = gamma_chirp.process()

figure()
imshow(flipud(gamma_chirp_mon.T), aspect='auto')    
show()    



'''
Example of using :class:`RemoteControlServer` and :class:`RemoteControlClient`
to control a simulation as it runs in Brian.

After running this script, run remotecontrolclient.py or paste the code from
that script into an IPython shell for interactive control.
'''
from brian import *

eqs = '''
dV/dt = (I-V)/(10*ms)+0.1*xi*(2/(10*ms))**.5 : 1
I : 1
'''

G = NeuronGroup(3, eqs, reset=0, threshold=1)
M = RecentStateMonitor(G, 'V', duration=50*ms)

server = RemoteControlServer()

run(1e10*second)



#import brian_no_units
from brian import *
from brian import optimiser
from scipy import weave
try:
    import pycuda
    import pycuda.autoinit as autoinit
    import pycuda.driver as drv
    import pycuda.compiler as compiler
    from pycuda import gpuarray
    from buffering import *
except ImportError:
    log_warn('brian.experimental.cuda.gpucodegen', 'Cannot import pycuda')
import time
from brian.experimental.codegen.rewriting import rewrite_to_c_expression

__all__ = ['GPUNonlinearStateUpdater', 'UserControlledGPUNonlinearStateUpdater',
           'GPUNeuronGroup']

#DEBUG_BUFFER_CACHE = False

class GPUNonlinearStateUpdater(NonlinearStateUpdater):
    def __init__(self, eqs, clock=None, freeze=False, precision='double', maxblocksize=512, forcesync=False):
        NonlinearStateUpdater.__init__(self, eqs, clock, compile=False, freeze=freeze)
        self.precision = precision
        if self.precision == 'double':
            self.precision_dtype = float64
        else:
            self.precision_dtype = float32
        self.clock_dt = float(guess_clock(clock).dt)
        self.code_gpu = self.generate_forward_euler_code()
        self.maxblocksize = maxblocksize
        self.forcesync = forcesync
        self._prepared = False

    def generate_forward_euler_code(self):
        eqs = self.eqs
        M = len(eqs._diffeq_names)
        all_variables = eqs._eq_names + eqs._diffeq_names + eqs._alias.keys() + ['t']
        clines = '__global__ void stateupdate(int N, SCALAR t, SCALAR *S)\n'
        clines += '{\n'
        clines += '    int i = blockIdx.x * blockDim.x + threadIdx.x;\n'
        clines += '    if(i>=N) return;\n'
        for j, name in enumerate(eqs._diffeq_names):
            clines += '    int _index_' + name + ' = i+' + str(j) + '*N;\n'
        for j, name in enumerate(eqs._diffeq_names):
#            clines += '    SCALAR &' + name + ' = S[i+'+str(j)+'*N];\n'
            clines += '    SCALAR ' + name + ' = S[_index_' + name + '];\n'
        for j, name in enumerate(eqs._diffeq_names):
            namespace = eqs._namespace[name]
            expr = optimiser.freeze(eqs._string[name], all_variables, namespace)
            expr = rewrite_to_c_expression(expr)
            print expr
            if name in eqs._diffeq_names_nonzero:
                clines += '    SCALAR ' + name + '__tmp = ' + expr + ';\n'
        for name in eqs._diffeq_names_nonzero:
#            clines += '    '+name+' += '+str(self.clock_dt)+'*'+name+'__tmp;\n'
            clines += '    S[_index_' + name + '] = ' + name + '+' + str(self.clock_dt) + '*' + name + '__tmp;\n'
        clines += '}\n'
        clines = clines.replace('SCALAR', self.precision)
        self.gpu_mod = compiler.SourceModule(clines)
        self.gpu_func = self.gpu_mod.get_function("stateupdate")
        return clines

    def _prepare(self, P):
        blocksize = self.maxblocksize
        if len(P) < blocksize:
            blocksize = len(P)
        if len(P) % blocksize == 0:
            gridsize = len(P) / blocksize
        else:
            gridsize = len(P) / blocksize + 1
        self._prepared = True
        self.gpu_func.prepare((int32, self.precision_dtype, 'i'), (blocksize, 1, 1))
        self._S_gpu_addr = P._S.gpu_pointer
        self._gpu_N = int32(len(P))
        self._gpu_grid = (gridsize, 1)

    def __call__(self, P):
        if not self._prepared:
            self._prepare(P)
        P._S.sync_to_gpu()
        self.gpu_func.prepared_call(self._gpu_grid, self._gpu_N, self.precision_dtype(P.clock._t), self._S_gpu_addr)
        P._S.changed_gpu_data()
        if self.forcesync:
            P._S.sync_to_cpu()
            P._S.changed_cpu_data()


class UserControlledGPUNonlinearStateUpdater(GPUNonlinearStateUpdater):
    def __init__(self, eqs, clock=None, freeze=False, precision='double', maxblocksize=512, gpu_to_cpu_vars=None, cpu_to_gpu_vars=None):
        GPUNonlinearStateUpdater.__init__(self, eqs, clock=clock, freeze=freeze, precision=precision, maxblocksize=maxblocksize, forcesync=False)
        self.gpu_to_cpu_vars = gpu_to_cpu_vars
        self.cpu_to_gpu_vars = cpu_to_gpu_vars

    def _prepare(self, P):
        GPUNonlinearStateUpdater._prepare(self, P)
        if isinstance(P._S._gpu_arr, pycuda.gpuarray.GPUArray):
            self._gpuoffset = int(P._S._gpu_arr.gpudata)
        elif isinstance(P._S._gpu_arr, pycuda.driver.DeviceAllocation):
            self._gpuoffset = int(P._S._gpu_arr)
        self._cpuflat = array(P._S, copy=False)
        self._cpuflat.shape = self._cpuflat.size

    def __call__(self, P):
        if not self._prepared:
            self._prepare(P)
        # copy from CPU to GPU
        for i, j, k in self.cpu_to_gpu_vars:
            pycuda.driver.memcpy_htod(self._gpuoffset + i, self._cpuflat[j:k])
        self.gpu_func.prepared_call(self._gpu_grid, self._gpu_N, self.precision_dtype(P.clock._t), self._S_gpu_addr)
        # copy from GPU back to CPU
        for i, j, k in self.gpu_to_cpu_vars:
            pycuda.driver.memcpy_dtoh(self._cpuflat[j:k], self._gpuoffset + i)
        P._S._cpu_data_changed = False # override any buffering
        P._S._gpu_data_changed = False # override any buffering


class GPUNeuronGroup(NeuronGroup):
    '''
    Neuron group which performs numerical integration on the GPU.
    
    .. warning::
        This class is still experimental, not supported and subject to change
        in future versions of Brian.
        
    Initialised with arguments as for :class:`NeuronGroup` and additionally:
    
    ``precision='double'``
        The GPU scalar precision to use, older models can only use
        ``precision='float'``.
    ``maxblocksize=512``
        If GPU compilation fails, reduce this value.
    ``forcesync=False``
        Whether or not to force copying of state variables to and from the
        GPU each time step. This is slow, so it is better to specify precisely
        which variables should be copied to and from using ``gpu_to_cpu_vars``
        and ``cpu_to_gpu_vars``.
    ``pagelocked_mem=True``
        Whether to store state variables in pagelocked memory on the CPU,
        which makes copying data to/from the GPU twice as fast.
    ``cpu_to_gpu_vars=None``, ``gpu_to_cpu_vars=None``
        Which variables should be copied each time step from the CPU to the GPU
        (before state update) and from the GPU to the CPU (after state update).
        
    The point of the copying of variables to and from the GPU is that the GPU
    maintains a separate memory from the CPU, and so changes made on either the
    CPU or GPU won't automatically be reflected in the other. Since only
    numerical integration is done on the GPU, any state variable that is
    modified by incoming synapses, for example, should be copied to and from
    the GPU each time step. In addition, any variables used for thresholding
    or resetting need to be appropriately copied (GPU->CPU for thresholding, and
    both for resetting).
    '''
    def __init__(self, N, model, threshold=None, reset=NoReset(),
                 init=None, refractory=0 * msecond, level=0,
                 clock=None, order=1, implicit=False, unit_checking=True,
                 max_delay=0 * msecond, compile=False, freeze=False, method=None,
                 precision='double', maxblocksize=512, forcesync=False, pagelocked_mem=True,
                 gpu_to_cpu_vars=None, cpu_to_gpu_vars=None):
        eqs = model
        eqs.prepare()
        NeuronGroup.__init__(self, N, eqs, threshold=threshold, reset=reset,
                             init=init, refractory=refractory, level=level,
                             clock=clock, order=order, compile=compile, freeze=freeze, method=method)
        self.precision = precision
        if self.precision == 'double':
            self.precision_dtype = float64
            self.precision_nbytes = 8
        else:
            self.precision_dtype = float32
            self.precision_nbytes = 4
        self.clock = guess_clock(clock)
        if gpu_to_cpu_vars is None and cpu_to_gpu_vars is None:
            self._state_updater = GPUNonlinearStateUpdater(eqs, clock=self.clock, precision=precision, maxblocksize=maxblocksize,
                                                           forcesync=forcesync)
        else:
            cpu_to_gpu_vars = [(self.get_var_index(var) * len(self) * self.precision_nbytes,
                                self.get_var_index(var) * len(self),
                                (self.get_var_index(var) + 1) * len(self)) for var in cpu_to_gpu_vars]
            gpu_to_cpu_vars = [(self.get_var_index(var) * len(self) * self.precision_nbytes,
                                self.get_var_index(var) * len(self),
                                (self.get_var_index(var) + 1) * len(self)) for var in gpu_to_cpu_vars]
            self._state_updater = UserControlledGPUNonlinearStateUpdater(eqs, clock=self.clock, precision=precision, maxblocksize=maxblocksize,
                                                           gpu_to_cpu_vars=gpu_to_cpu_vars, cpu_to_gpu_vars=cpu_to_gpu_vars)
        if pagelocked_mem:
            self._S = GPUBufferedArray(drv.pagelocked_zeros(self._S.shape, dtype=self.precision_dtype))
        else:
            self._S = GPUBufferedArray(array(self._S, dtype=self.precision_dtype))
        self._gpuneurongroup_init_finished = True

    def copyvar_cpu_to_gpu(self, var):
        i, j, k = (self.get_var_index(var) * len(self) * self.precision_nbytes,
                   self.get_var_index(var) * len(self),
                   (self.get_var_index(var) + 1) * len(self))
        pycuda.driver.memcpy_htod(self._state_updater._gpuoffset + i, self._state_updater._cpuflat[j:k])

    def copyvar_gpu_to_cpu(self, var):
        i, j, k = (self.get_var_index(var) * len(self) * self.precision_nbytes,
                   self.get_var_index(var) * len(self),
                   (self.get_var_index(var) + 1) * len(self))
        pycuda.driver.memcpy_dtoh(self._state_updater._cpuflat[j:k], self._state_updater._gpuoffset + i)

    def __setattr__(self, name, val):
        try:
            self._gpuneurongroup_init_finished
            NeuronGroup.__setattr__(self, name, val)
            if name in self.var_index:
                self._S.changed_cpu_data()
        except AttributeError:
            object.__setattr__(self, name, val)

if __name__ == '__main__':

    from brian.experimental.ccodegen import AutoCompiledNonlinearStateUpdater
    set_global_preferences(usecodegen=False)

    #duration = 10*second
    #N = 1000
    #domonitor = False

    duration = 1000 * ms
    N = 100
    domonitor = False
    showfinal = False
    forcesync = True
    method = 'gpu' # methods are 'c', 'python' and 'gpu'

    if drv.get_version() == (2, 0, 0): # cuda version
        precision = 'float'
    elif drv.get_version() > (2, 0, 0):
        precision = 'double'
    else:
        raise Exception, "CUDA 2.0 required"
    #precision = 'float'
    import buffering
    buffering.DEBUG_BUFFER_CACHE = False

#    eqs = Equations('''
#    #dV/dt = -V*V/(10*ms) : 1
#    dV/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
#    #dV/dt = -V*V*V*V*V/(100*ms) : 1
#    #dW/dt = -W*W*W*W*W/(100*ms) : 1
#    #dV/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
#    #dW/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
#    #dW2/dt = cos(2*pi*t/(100*ms))/(10*ms) : 1
#    #dV/dt = h/(10*ms) : 1
#    #h = -V*V : 1
#    ''')
    #eqs = Equations('\n'.join('dv'+str(i)+'/dt=-v'+str(i)+'/second:1' for i in range(20))) #10 works 11 is too much
    #print eqs

#    taum=20*ms
#    taue=5*ms
#    taui=10*ms
#    Vt=-50*mV
#    Vr=-60*mV
#    El=-49*mV
#    
#    eqs= Equations('''
#    dV/dt  = (ge+gi-(V-El))/taum : volt
#    dge/dt = -ge/taue : volt
#    dgi/dt = -gi/taui : volt
#    ''')
    
    from brian.library.ionic_currents import *

    El = 10.6 * mV
    EK = -12 * mV
    ENa = 120 * mV
    eqs = MembraneEquation(1 * uF) + leak_current(.3 * msiemens, El)
    eqs += K_current_HH(36 * msiemens, EK) + Na_current_HH(120 * msiemens, ENa)
    eqs += Current('I:amp')
    #eqs.prepare()
    #
    #for n in eqs._string.keys():
    #    eqs._string[n] = rewrite_to_c_expression(eqs._string[n])
    #print eqs


    if method == 'gpu':
        G = GPUNeuronGroup(N, eqs, precision=precision, maxblocksize=256, forcesync=forcesync)

        print 'GPU loop code:'
        print G._state_updater.code_gpu
        gf = G._state_updater.gpu_func
        print '(lmem, smem, registers) = ', (gf.local_size_bytes, gf.shared_size_bytes, gf.num_regs)
        devdata = pycuda.tools.DeviceData()
        orec = pycuda.tools.OccupancyRecord(devdata, 256)
        print 'tb_per_mp', orec.tb_per_mp
        print 'limited_by', orec.limited_by
        print 'warps_per_mp', orec.warps_per_mp
        print 'occupancy', orec.occupancy
    elif method == 'c':
        G = NeuronGroup(N, eqs, compile=True, freeze=True)
        su = AutoCompiledNonlinearStateUpdater(eqs, G.clock, freeze=True)
        G._state_updater = su
    elif method == 'python':
        #G = NeuronGroup(N, eqs, freeze=True)#, compile=True, freeze=True)
        G = NeuronGroup(N, eqs, compile=True, freeze=True)

    G.V = 1

    if domonitor:
        M = StateMonitor(G, 'V', record=True)

    start = time.time()
    run(duration)
    autoinit.context.synchronize()
    print method, 'code:', (time.time() - start)
    if domonitor: M_V = M[0]

    if domonitor:
        plot(M.times, M_V)
    if showfinal:
        figure()
        plot(G.V)
    if domonitor or showfinal:
        show()



'''
Implementation example of the compressive gammachirp auditory filter as
described in Irino, T. and Patterson R., "A compressive gammachirp auditory
filter for both physiological and psychophysical data", JASA 2001.

A class called :class:`~brian.hears.DCGC` implementing this model is available
in the library.

Technical implementation details and notation can be found in Irino, T. and
Patterson R., "A Dynamic Compressive Gammachirp Auditory Filterbank",
IEEE Trans Audio Speech Lang Processing.
'''
from brian import *
from brian.hears import *

simulation_duration = 50*ms
samplerate = 50*kHz
level = 50*dB # level of the input sound in rms dB SPL
sound = whitenoise(simulation_duration, samplerate).ramp()
sound = sound.atlevel(level)

nbr_cf = 50 # number of centre frequencies
# center frequencies with a spacing following an ERB scale
cf = erbspace(100*Hz, 1000*Hz, nbr_cf)

c1 = -2.96 #glide slope of the first filterbank
b1 = 1.81  #factor determining the time constant of the first filterbank
c2 = 2.2   #glide slope of the second filterbank
b2 = 2.17  #factor determining the time constant of the second filterbank

order_ERB = 4
ERBrate = 21.4*log10(4.37*cf/1000+1)
ERBwidth = 24.7*(4.37*cf/1000 + 1)
ERBspace = mean(diff(ERBrate))

# the filter coefficients are updated every update_interval (here in samples)
update_interval = 1

                  

#bank of passive gammachirp filters. As the control path uses the same passive
#filterbank than the signal path (but shifted in frequency)
#this filterbank is used by both pathway.
pGc = LogGammachirp(sound, cf, b=b1, c=c1)

fp1 = cf + c1*ERBwidth*b1/order_ERB #centre frequency of the signal path

#### Control Path ####

#the first filterbank in the control path consists of gammachirp filters
#value of the shift in ERB frequencies of the control path with respect to the signal path
lct_ERB = 1.5
n_ch_shift = round(lct_ERB/ERBspace) #value of the shift in channels
#index of the channel of the control path taken from pGc
indch1_control = minimum(maximum(1, arange(1, nbr_cf+1)+n_ch_shift), nbr_cf).astype(int)-1 
fp1_control = fp1[indch1_control]
#the control path bank pass filter uses the channels of pGc indexed by indch1_control
pGc_control = RestructureFilterbank(pGc, indexmapping=indch1_control)

#the second filterbank in the control path consists of fixed asymmetric compensation filters
frat_control = 1.08
fr2_control = frat_control*fp1_control
asym_comp_control = AsymmetricCompensation(pGc_control, fr2_control, b=b2, c=c2)

#definition of the pole of the asymmetric comensation filters
p0 = 2
p1 = 1.7818*(1-0.0791*b2)*(1-0.1655*abs(c2))
p2 = 0.5689*(1-0.1620*b2)*(1-0.0857*abs(c2))
p3 = 0.2523*(1-0.0244*b2)*(1+0.0574*abs(c2))
p4 = 1.0724

#definition of the parameters used in the control path output levels computation
#(see IEEE paper for details)
decay_tcst = .5*ms
order = 1.
lev_weight = .5
level_ref = 50.
level_pwr1 = 1.5
level_pwr2 = .5
RMStoSPL = 30.
frat0 = .2330
frat1 = .005 
exp_deca_val = exp(-1/(decay_tcst*samplerate)*log(2))
level_min = 10**(-RMStoSPL/20)

#definition of the controller class. What is does it take the outputs of the
#first and second fitlerbanks of the control filter as input, compute an overall
#intensity level for each frequency channel. It then uses those level to update
#the filter coefficient of its target, the asymmetric compensation filterbank of
#the signal path.
class CompensensationFilterUpdater(object): 
    def __init__(self, target):
        self.target = target
        self.level1_prev = -100
        self.level2_prev = -100
        
    def __call__(self, *input):
         value1 = input[0][-1,:]
         value2 = input[1][-1,:]
         #the current level value is chosen as the max between the current
         #output and the previous one decreased by a decay
         level1 = maximum(maximum(value1, 0), self.level1_prev*exp_deca_val) 
         level2 = maximum(maximum(value2, 0), self.level2_prev*exp_deca_val)

         self.level1_prev = level1 #the value is stored for the next iteration
         self.level2_prev = level2
         #the overall intensity is computed between the two filterbank outputs
         level_total = lev_weight*level_ref*(level1/level_ref)**level_pwr1+\
                   (1-lev_weight)*level_ref*(level2/level_ref)**level_pwr2
         #then it is converted in dB
         level_dB = 20*log10(maximum(level_total, level_min))+RMStoSPL
         #the frequency factor is calculated           
         frat = frat0 + frat1*level_dB
         #the centre frequency of the asymmetric compensation filters are updated       
         fr2 = fp1*frat
         coeffs = asymmetric_compensation_coeffs(samplerate, fr2,
                        self.target.filt_b, self.target.filt_a, b2, c2,
                        p0, p1, p2, p3, p4)
         self.target.filt_b, self.target.filt_a = coeffs                 

#### Signal Path ####
#the signal path consists of the passive gammachirp filterbank pGc previously
#defined followed by a asymmetric compensation filterbank
fr1 = fp1*frat0
varyingfilter_signal_path = AsymmetricCompensation(pGc, fr1, b=b2, c=c2)
updater = CompensensationFilterUpdater(varyingfilter_signal_path)
 #the controler which takes the two filterbanks of the control path as inputs
 #and the varying filter of the signal path as target is instantiated
control = ControlFilterbank(varyingfilter_signal_path,
                            [pGc_control, asym_comp_control],
                            varyingfilter_signal_path, updater, update_interval)  

#run the simulation
#Remember that the controler are at the end of the chain and the output of the
#whole path comes from them
signal = control.process() 

figure()
imshow(flipud(signal.T), aspect='auto')    
show()
